@@ -1,5 +1,5 @@
 /* data.c: Application data management routines for gzochid
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -31,6 +31,11 @@
 #include "tx.h"
 #include "util.h"
 
+#ifdef G_LOG_DOMAIN
+#undef G_LOG_DOMAIN
+#endif /* G_LOG_DOMAIN */
+#define G_LOG_DOMAIN "gzochid.data"
+
 struct _gzochid_data_oid_block_state
 {
   guint64 first;
@@ -89,13 +94,13 @@ create_transaction_context (gzochid_application_context *app_context,
   tx_context->context = app_context;
 
   if (timeout != NULL)
-    tx_context->transaction = APP_STORAGE_INTERFACE (app_context)
+    tx_context->transaction = app_context->storage_engine_interface
       ->transaction_begin_timed (app_context->storage_context, *timeout);
-  else tx_context->transaction = APP_STORAGE_INTERFACE (app_context)
+  else tx_context->transaction = app_context->storage_engine_interface
 	 ->transaction_begin (app_context->storage_context);
 
   tx_context->oids_to_references = g_hash_table_new_full 
-    (g_int64_hash, g_int64_equal, free, NULL);
+    (g_int64_hash, g_int64_equal, g_free, NULL);
   tx_context->ptrs_to_references = g_hash_table_new 
     (g_direct_hash, g_direct_equal);
 
@@ -137,7 +142,7 @@ flush_reference (gzochid_data_managed_reference *reference,
   GError *err = NULL;
 
   guint64 encoded_oid = gzochid_util_encode_oid (reference->oid);
-
+  
   switch (reference->state)
     {
     case GZOCHID_MANAGED_REFERENCE_STATE_EMPTY:
@@ -163,15 +168,16 @@ flush_reference (gzochid_data_managed_reference *reference,
 	  return FALSE;
 	}
 
-      APP_STORAGE_INTERFACE (context->context)->transaction_put
+      context->context->storage_engine_interface->transaction_put
 	(context->transaction, context->context->oids,
 	 (char *) &encoded_oid, sizeof (guint64),
 	 (char *) out->data, out->len);
 
-      gzochid_application_event_dispatch
+      gzochid_event_dispatch
 	(context->context->event_source,
-	 gzochid_application_data_event_new (BYTES_WRITTEN, out->len));
-
+	 g_object_new (GZOCHID_TYPE_DATA_EVENT,
+		       "type", BYTES_WRITTEN, "bytes", out->len, NULL));
+      
       g_byte_array_unref (out);
       
       if (context->transaction->rollback)
@@ -194,8 +200,6 @@ data_prepare (gpointer data)
   GList *references = g_hash_table_get_values (context->oids_to_references);
   GList *reference_ptr = references;
   gboolean flush_failed = FALSE;
-  gzochid_storage_engine_interface *iface = 
-    APP_STORAGE_INTERFACE (context->context);
 
   while (reference_ptr != NULL)
     {
@@ -212,7 +216,9 @@ data_prepare (gpointer data)
   if (flush_failed)
     return FALSE;
 
-  iface->transaction_prepare (context->transaction);
+  context->context->storage_engine_interface
+    ->transaction_prepare (context->transaction);
+
   if (context->transaction->rollback)
     return FALSE;
 
@@ -245,10 +251,10 @@ static void
 data_commit (gpointer data)
 {
   gzochid_data_transaction_context *context = data;
-  gzochid_storage_engine_interface *iface = 
-    APP_STORAGE_INTERFACE (context->context);
 
-  iface->transaction_commit (context->transaction);
+  context->context->storage_engine_interface
+    ->transaction_commit (context->transaction);
+
   finalize_references (context);
   transaction_context_free (context);
 }
@@ -264,10 +270,9 @@ data_rollback (gpointer data)
 
   if (context != NULL)
     {
-      gzochid_storage_engine_interface *iface = 
-	APP_STORAGE_INTERFACE (context->context);
+      context->context->storage_engine_interface
+	->transaction_rollback (context->transaction);
 
-      iface->transaction_rollback (context->transaction);
       finalize_references (context);
       transaction_context_free (context);
     }
@@ -291,12 +296,18 @@ reserve_oids (gzochid_application_context *context)
     }
   else
     {
+      GError *err = NULL;
       gzochid_data_oids_block new_block;
 
-      assert (gzochid_oids_reserve_block
-	      (context->oid_strategy, &new_block, NULL));
-
-      block_state = create_oid_block_state (&new_block);
+      if (gzochid_oids_reserve_block (context->oid_strategy, &new_block, &err))
+	block_state = create_oid_block_state (&new_block);
+      else
+	{
+	  g_debug ("Failed to reserve oids: %s", err->message);
+	  gzochid_transaction_mark_for_rollback
+	    (&data_participant, err->code == GZOCHID_OIDS_ERROR_TRANSACTION);
+	  g_error_free (err);
+	}
     }
 
   g_mutex_unlock (&context->free_oids_lock);
@@ -308,7 +319,7 @@ get_binding (gzochid_data_transaction_context *context, char *name,
 	     GError **err)
 {
   size_t oid_bytes_len = 0;
-  char *oid_bytes = APP_STORAGE_INTERFACE (context->context)->transaction_get 
+  char *oid_bytes = context->context->storage_engine_interface->transaction_get 
     (context->transaction, context->context->names, name, strlen (name) + 1, 
      &oid_bytes_len);
 
@@ -350,7 +361,7 @@ set_binding (gzochid_data_transaction_context *context, char *name, guint64 oid,
 {
   guint64 encoded_oid = gzochid_util_encode_oid (oid);
   
-  APP_STORAGE_INTERFACE (context->context)->transaction_put 
+  context->context->storage_engine_interface->transaction_put 
     (context->transaction, context->context->names, name, strlen (name) + 1, 
      (char *) &encoded_oid, sizeof (guint64));
 
@@ -379,12 +390,21 @@ create_empty_reference (gzochid_application_context *context, guint64 oid,
   return reference;
 }
 
-static int 
-next_object_id (gzochid_data_transaction_context *context, guint64 *oid)
+static gboolean
+next_object_id (gzochid_data_transaction_context *context, guint64 *oid,
+		GError **err)
 {
   if (context->free_oids == NULL)
     context->free_oids = reserve_oids (context->context);
 
+  if (context->free_oids == NULL)
+    {
+      g_set_error
+	(err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION,
+	 "Failed to compute next object id.");
+      return FALSE;
+    }
+  
   *oid = context->free_oids->next;
   context->free_oids->next += 1;
 
@@ -393,26 +413,33 @@ next_object_id (gzochid_data_transaction_context *context, guint64 *oid)
       context->used_oid_blocks = g_list_append 
 	(context->used_oid_blocks, context->free_oids);
       context->free_oids = reserve_oids (context->context);
+
+      if (context->free_oids == NULL)
+	{
+	  g_set_error
+	    (err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION,
+	     "Failed to compute next object id.");
+	  return FALSE;
+	}
     }
 
-  return 0;
+  return TRUE;
 }
 
 gzochid_data_managed_reference *
 create_new_reference (gzochid_application_context *context, void *ptr, 
-		      gzochid_io_serialization *serialization)
+		      gzochid_io_serialization *serialization, GError **err)
 {
+  guint64 oid;
+  GError *local_err = NULL;
   gzochid_data_transaction_context *tx_context = 
     gzochid_transaction_context (&data_participant);
-  gzochid_data_managed_reference *reference = 
-    calloc (1, sizeof (gzochid_data_managed_reference));
-  
-  reference->context = context;
-  
-  next_object_id (tx_context, &reference->oid);
-  reference->state = GZOCHID_MANAGED_REFERENCE_STATE_NEW;
-  reference->serialization = serialization;
-  reference->obj = ptr;
+    
+  if (!next_object_id (tx_context, &oid, &local_err))
+    {
+      g_propagate_error (err, local_err);
+      return NULL;
+    }
 
   /*
     Once this function returns, callers will have an oid of the object that will
@@ -424,14 +451,41 @@ create_new_reference (gzochid_application_context *context, void *ptr,
   
   if (!tx_context->transaction->rollback)
     {
-      guint64 encoded_oid = gzochid_util_encode_oid (reference->oid);
+      gzochid_data_managed_reference *reference = 
+	calloc (1, sizeof (gzochid_data_managed_reference));
+      guint64 encoded_oid;
+
+      reference->oid = oid;
+      reference->state = GZOCHID_MANAGED_REFERENCE_STATE_NEW;
+      reference->serialization = serialization;
+      reference->obj = ptr;
+
+      encoded_oid = gzochid_util_encode_oid (reference->oid);
       
-      APP_STORAGE_INTERFACE (context)->transaction_get_for_update
+      context->storage_engine_interface->transaction_get_for_update
 	(tx_context->transaction, tx_context->context->oids,
 	 (char *) &encoded_oid, sizeof (guint64), NULL);
+
+      if (tx_context->transaction->rollback)	
+	{
+	  gzochid_transaction_mark_for_rollback 
+	    (&data_participant, tx_context->transaction->should_retry);
+	  g_set_error 
+	    (err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION, 
+	     "Transaction rolled back while creating new reference.");
+
+	  free (reference);
+	  return NULL;
+	}
+      else return reference;
+    }
+  else
+    {
+      g_set_error 
+	(err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION, 
+	 "Attempted reference creation after transaction marked for rollback.");
+      return NULL;
     }
-  
-  return reference;
 }
 
 static gzochid_data_managed_reference *
@@ -445,12 +499,10 @@ get_reference_by_oid (gzochid_application_context *context, guint64 oid,
 
   if (reference == NULL)
     {
-      guint64 *key = malloc (sizeof (guint64));
-
-      *key = oid;
-      
       reference = create_empty_reference (context, oid, serialization);
-      g_hash_table_insert (tx_context->oids_to_references, key, reference);
+      g_hash_table_insert
+	(tx_context->oids_to_references, g_memdup (&oid, sizeof (guint64)),
+	 reference);
     }
 
   return reference;
@@ -467,30 +519,19 @@ get_reference_by_ptr (gzochid_application_context *context, void *ptr,
 
   if (reference == NULL)
     {
-      guint64 *key = NULL;
-      
-      reference = create_new_reference (context, ptr, serialization);
+      GError *local_err = NULL;
 
-      assert (g_hash_table_lookup 
-	      (tx_context->oids_to_references, &reference->oid) == NULL);
-      assert (g_hash_table_lookup 
-	      (tx_context->ptrs_to_references, ptr) == NULL);
+      reference = create_new_reference
+	(context, ptr, serialization, &local_err);
 
-      key = malloc (sizeof (guint64));
-      *key = reference->oid;
-      
-      g_hash_table_insert (tx_context->oids_to_references, key, reference);
-      g_hash_table_insert (tx_context->ptrs_to_references, ptr, reference);
-
-      if (tx_context->transaction->rollback)
+      if (reference != NULL)
 	{
-	  gzochid_transaction_mark_for_rollback 
-	    (&data_participant, tx_context->transaction->should_retry);
-	  g_set_error 
-	    (err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION, 
-	     "Transaction rolled back while creating new reference.");
-	  return NULL;
+	  g_hash_table_insert
+	    (tx_context->oids_to_references,
+	     g_memdup (&reference->oid, sizeof (guint64)), reference);
+	  g_hash_table_insert (tx_context->ptrs_to_references, ptr, reference);
 	}
+      else g_propagate_error (err, local_err);
     }
   
   return reference;
@@ -525,10 +566,11 @@ dereference (gzochid_data_transaction_context *context,
   encoded_oid = gzochid_util_encode_oid (reference->oid);
   
   if (for_update)
-    data = APP_STORAGE_INTERFACE (context->context)->transaction_get_for_update
+    data = context->context->storage_engine_interface
+      ->transaction_get_for_update
       (context->transaction, context->context->oids, (char *) &encoded_oid,
        sizeof (guint64), &data_len);
-  else data = APP_STORAGE_INTERFACE (context->context)->transaction_get
+  else data = context->context->storage_engine_interface->transaction_get
 	 (context->transaction, context->context->oids, (char *) &encoded_oid, 
 	  sizeof (guint64), &data_len);
 
@@ -548,12 +590,12 @@ dereference (gzochid_data_transaction_context *context,
   else 
     {
       GError *local_err = NULL;
-      guint64 *key = malloc (sizeof (guint64));
       
-      gzochid_application_event_dispatch
+      gzochid_event_dispatch
 	(context->context->event_source,
-	 gzochid_application_data_event_new (BYTES_READ, data_len));
-
+	 g_object_new (GZOCHID_TYPE_DATA_EVENT,
+		       "type", BYTES_READ, "bytes", data_len, NULL));
+      
       in = g_byte_array_sized_new (data_len);
       g_byte_array_append (in, (unsigned char *) data, data_len);
       free (data);
@@ -567,9 +609,9 @@ dereference (gzochid_data_transaction_context *context,
 	reference->state = GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED;
       else reference->state = GZOCHID_MANAGED_REFERENCE_STATE_NOT_MODIFIED;
 
-      *key = reference->oid;
-      
-      g_hash_table_insert (context->oids_to_references, key, reference);
+      g_hash_table_insert
+	(context->oids_to_references,
+	 g_memdup (&reference->oid, sizeof (guint64)), reference);
       g_hash_table_insert 
 	(context->ptrs_to_references, reference->obj, reference);
 
@@ -593,7 +635,7 @@ remove_object (gzochid_data_transaction_context *context, guint64 oid,
   guint64 encoded_oid = gzochid_util_encode_oid (oid);
   
   g_debug ("Removing reference '%" G_GUINT64_FORMAT "'.", oid);
-  ret = APP_STORAGE_INTERFACE (context->context)->transaction_delete 
+  ret = context->context->storage_engine_interface->transaction_delete 
     (context->transaction, context->context->oids, (char *) &encoded_oid, 
      sizeof (guint64));
 
@@ -719,7 +761,6 @@ gzochid_data_next_binding_oid (gzochid_application_context *context, char *key,
   GError *local_err = NULL;
   gzochid_data_transaction_context *tx_context = 
     join_transaction (context, &local_err);
-  gzochid_storage_engine_interface *iface = APP_STORAGE_INTERFACE (context);
 
   if (local_err != NULL)
     {
@@ -727,7 +768,7 @@ gzochid_data_next_binding_oid (gzochid_application_context *context, char *key,
       return NULL;
     }
 
-  next_key = iface->transaction_next_key 
+  next_key = context->storage_engine_interface->transaction_next_key 
     (tx_context->transaction, tx_context->context->names, key, 
      strlen (key) + 1, NULL);
   if (tx_context->transaction->rollback)
@@ -789,7 +830,7 @@ gzochid_data_remove_binding (gzochid_application_context *context, char *name,
       return;
     }
 
-  ret = APP_STORAGE_INTERFACE (context)->transaction_delete 
+  ret = context->storage_engine_interface->transaction_delete 
     (tx_context->transaction, tx_context->context->names, name, 
      strlen (name) + 1);
 
@@ -939,7 +980,7 @@ gzochid_data_mark (gzochid_application_context *context,
       g_debug ("Marking reference '%" G_GUINT64_FORMAT "' for update.",
 	       reference->oid);
 
-      data = APP_STORAGE_INTERFACE (context)->transaction_get_for_update
+      data = context->storage_engine_interface->transaction_get_for_update
 	(tx_context->transaction, tx_context->context->oids,
 	 (char *) &encoded_oid, sizeof (guint64), NULL);
    
