@@ -1,5 +1,5 @@
 /* dataserver.c: Data server for gzochi-metad
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -23,20 +23,31 @@
 #include <string.h>
 
 #include "config.h"
-#include "dataserver.h"
 #include "dataserver-protocol.h"
+#include "dataserver.h"
 #include "gzochid-storage.h"
+#include "httpd.h"
 #include "lock.h"
-#include "oids.h"
+#include "log.h"
 #include "oids-storage.h"
+#include "oids.h"
+#include "resolver.h"
 #include "socket.h"
-#include "storage.h"
 #include "storage-mem.h"
+#include "storage.h"
+#include "util.h"
+
+#ifdef G_LOG_DOMAIN
+#undef G_LOG_DOMAIN
+#endif /* G_LOG_DOMAIN */
+#define G_LOG_DOMAIN "gzochi-metad.dataserver"
 
 #ifndef GZOCHID_STORAGE_ENGINE_DIR
 #define GZOCHID_STORAGE_ENGINE_DIR "./storage"
 #endif /* GZOCHID_STORAGE_ENGINE_DIR */
 
+#define LOCK_ACCESS(for_write) (for_write ? "r/w" : "read")
+
 /* A store with an associated lock table. */
 
 struct _gzochi_metad_dataserver_lockable_store
@@ -78,10 +89,21 @@ gzochi_metad_dataserver_application_store;
 
 struct _GzochiMetadDataServer
 {
-  GObject parent_instance;
+  GObject parent_instance; /* The parent struct, for casting. */
 
-  GHashTable *configuration; /* The dataserver configuration table. */
+  GzochidConfiguration *configuration; /* The global configuration object. */
 
+  /* The data client configuration table; extracted from the global 
+     configuration object and cached for convenience. */
+
+  GHashTable *data_configuration;
+
+  /* The global resolution context; used for getting a (temporary) handle to
+     the meta server's embedded HTTP server (if available) to extract the base
+     URL to send to the client as part of the login response. */
+  
+  GzochidResolutionContext *resolution_context;
+  
   /* The socket server for the data server. */
   
   GzochidSocketServer *socket_server; 
@@ -91,14 +113,6 @@ struct _GzochiMetadDataServer
   /* Mapping application name to `gzochi_metad_dataserver_application_store'. */
 
   GHashTable *application_stores; 
-
-  gzochid_server_socket *server_socket; /* The dataserver's server socket. */
-
-  /* The port on which the server listens. This may be zero to indicate the 
-     server should listen on any available port, and will be set to the port 
-     assigned by the operating system. */
-
-  guint port; 
 };
 
 #define STORAGE_INTERFACE(server) server->storage_engine->interface
@@ -108,12 +122,22 @@ G_DEFINE_TYPE (GzochiMetadDataServer, gzochi_metad_data_server, G_TYPE_OBJECT);
 enum gzochi_metad_data_server_properties
   {
     PROP_CONFIGURATION = 1,
+    PROP_RESOLUTION_CONTEXT,
     PROP_SOCKET_SERVER,
     N_PROPERTIES
   };
 
 static GParamSpec *obj_properties[N_PROPERTIES] = { NULL };
 
+static void
+gzochi_metad_data_server_constructed (GObject *object)
+{
+  GzochiMetadDataServer *data_server = GZOCHI_METAD_DATA_SERVER (object);
+
+  data_server->data_configuration = gzochid_configuration_extract_group
+    (data_server->configuration, "data");  
+}
+
 static void
 gzochi_metad_data_server_set_property (GObject *object, guint property_id,
 				       const GValue *value, GParamSpec *pspec)
@@ -123,18 +147,17 @@ gzochi_metad_data_server_set_property (GObject *object, guint property_id,
   switch (property_id)
     {
     case PROP_CONFIGURATION:
-      self->configuration = gzochid_configuration_extract_group
-	(GZOCHID_CONFIGURATION (g_value_get_object (value)), "data");
-
-      self->port = gzochid_config_to_int
-	(g_hash_table_lookup (self->configuration, "server.port"), 9001);
-
+      self->configuration = g_object_ref (g_value_get_object (value));
       break;
 
+    case PROP_RESOLUTION_CONTEXT:
+      self->resolution_context = g_object_ref_sink (g_value_get_object (value));
+      break;
+      
     case PROP_SOCKET_SERVER:
       self->socket_server = g_object_ref (g_value_get_object (value));
       break;
-      
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -173,7 +196,7 @@ gzochi_metad_data_server_finalize (GObject *gobject)
 {
   GzochiMetadDataServer *server = GZOCHI_METAD_DATA_SERVER (gobject);
   
-  g_hash_table_destroy (server->configuration);
+  g_hash_table_destroy (server->data_configuration);
 
   g_hash_table_foreach_remove
     (server->application_stores, close_application_store, server);
@@ -188,6 +211,8 @@ gzochi_metad_data_server_finalize (GObject *gobject)
 	
 	free (server->storage_engine);
     }
+
+  G_OBJECT_CLASS (gzochi_metad_data_server_parent_class)->finalize (gobject);
 }
 
 static void
@@ -195,7 +220,11 @@ gzochi_metad_data_server_dispose (GObject *gobject)
 {
   GzochiMetadDataServer *server = GZOCHI_METAD_DATA_SERVER (gobject);
 
+  g_object_unref (server->configuration);
+  g_object_unref (server->resolution_context);
   g_object_unref (server->socket_server);
+
+  G_OBJECT_CLASS (gzochi_metad_data_server_parent_class)->dispose (gobject);
 }
 
 static void
@@ -203,17 +232,23 @@ gzochi_metad_data_server_class_init (GzochiMetadDataServerClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->constructed = gzochi_metad_data_server_constructed;
   object_class->dispose = gzochi_metad_data_server_dispose;
   object_class->finalize = gzochi_metad_data_server_finalize;
   object_class->set_property = gzochi_metad_data_server_set_property;
 
   obj_properties[PROP_CONFIGURATION] = g_param_spec_object
-    ("configuration", "gzochi meta server configuration",
-     "Set the meta server configuration", GZOCHID_TYPE_CONFIGURATION,
+    ("configuration", "config", "The global configuration object",
+     GZOCHID_TYPE_CONFIGURATION,
+     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
+
+  obj_properties[PROP_RESOLUTION_CONTEXT] = g_param_spec_object
+    ("resolution-context", "resolution-context",
+     "The global resolution context", GZOCHID_TYPE_RESOLUTION_CONTEXT,
      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
   
   obj_properties[PROP_SOCKET_SERVER] = g_param_spec_object
-    ("socket-server", "Socket server", "Set the socket server",
+    ("socket-server", "socket-server", "The global socket server",
      GZOCHID_TYPE_SOCKET_SERVER,
      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
 
@@ -224,17 +259,14 @@ gzochi_metad_data_server_class_init (GzochiMetadDataServerClass *klass)
 static void
 gzochi_metad_data_server_init (GzochiMetadDataServer *self)
 {
-  self->server_socket = gzochid_server_socket_new
-    ("Data server", gzochi_metad_dataserver_server_protocol, self);
   self->application_stores = g_hash_table_new_full
     (g_str_hash, g_str_equal, (GDestroyNotify) free, (GDestroyNotify) free);
-  self->port = 0;
 }
 
 void
 gzochi_metad_dataserver_start (GzochiMetadDataServer *self)
 {
-  if (g_hash_table_contains (self->configuration, "storage.engine"))
+  if (g_hash_table_contains (self->data_configuration, "storage.engine"))
     {
       const char *dir = NULL;
       const char *env = getenv ("GZOCHID_STORAGE_ENGINE_DIR");
@@ -244,12 +276,12 @@ gzochi_metad_dataserver_start (GzochiMetadDataServer *self)
       else 
         {
           const char *conf_dir = g_hash_table_lookup
-	    (self->configuration, "storage.engine.dir");
+	    (self->data_configuration, "storage.engine.dir");
           dir = conf_dir == NULL ? GZOCHID_STORAGE_ENGINE_DIR : conf_dir;
         }
 
       self->storage_engine = gzochid_storage_load_engine 
-        (dir, g_hash_table_lookup (self->configuration, "storage.engine"));
+        (dir, g_hash_table_lookup (self->data_configuration, "storage.engine"));
     }
   else g_message 
          ("No durable storage engine configured; memory engine will be used.");
@@ -263,9 +295,6 @@ FOR PRODUCTION USE.");
       self->storage_engine = calloc (1, sizeof (gzochid_storage_engine));
       self->storage_engine->interface = &gzochid_storage_engine_interface_mem;
     }
-  
-  gzochid_server_socket_listen
-    (self->socket_server, self->server_socket, self->port);
 }
 
 /* Switch on the specified store name to return either the oids store or the
@@ -274,7 +303,7 @@ FOR PRODUCTION USE.");
 
 static gzochi_metad_dataserver_lockable_store *
 get_lockable_store (gzochi_metad_dataserver_application_store *store,
-		    char *name, GError **err)
+		    const char *name, GError **err)
 {
   if (strcmp (name, "oids") == 0)
     return store->oids;
@@ -339,24 +368,29 @@ ensure_open_application_store (const GzochiMetadDataServer *server,
 
 gzochid_data_reserve_oids_response *
 gzochi_metad_dataserver_reserve_oids (GzochiMetadDataServer *server,
-				      guint node_id, char *app)
+				      guint node_id, const char *app)
 {
   gzochid_data_oids_block oids_block;
   gzochid_data_reserve_oids_response *response = NULL;
   gzochi_metad_dataserver_application_store *app_store =
     ensure_open_application_store (server, app);
 
+  gzochid_trace ("Node %d requested oid block for %s.", node_id, app);
+  
   assert (gzochid_oids_reserve_block
 	  (app_store->oid_strategy, &oids_block, NULL));
 
+  gzochid_trace ("Reserved block { %" G_GUINT64_FORMAT ", %d } for node %d/%s.",
+		 oids_block.block_start, oids_block.block_size, node_id, app);
+  
   response = gzochid_data_reserve_oids_response_new (app, &oids_block);
   return response;
 }
 
 gzochid_data_response *
 gzochi_metad_dataserver_request_value (GzochiMetadDataServer *server,
-				       guint node_id, char *app,
-				       char *store_name, GBytes *key,
+				       guint node_id, const char *app,
+				       const char *store_name, GBytes *key,
 				       gboolean for_write, GError **err)
 {
   gzochid_data_response *response = NULL;
@@ -374,6 +408,12 @@ gzochi_metad_dataserver_request_value (GzochiMetadDataServer *server,
       g_propagate_error (err, local_err);
       return NULL;
     }
+
+  if (gzochid_log_level_visible (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
+    GZOCHID_WITH_FORMATTED_BYTES
+      (key, buf, 33, gzochid_trace
+       ("Node %d requested %s lock on key %s/%s/%s.", node_id,
+	LOCK_ACCESS (for_write), app, store_name, buf));
   
   if (gzochid_lock_check_and_set
       (store->locks, node_id, key, for_write, &most_recent_lock))
@@ -400,15 +440,30 @@ gzochi_metad_dataserver_request_value (GzochiMetadDataServer *server,
 
       STORAGE_INTERFACE (server)->transaction_rollback (transaction);
 
+      if (gzochid_log_level_visible (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
+	GZOCHID_WITH_FORMATTED_BYTES
+	  (key, buf, 33, gzochid_trace
+	   ("Granted %s lock on key %s/%s/%s to node %d.",
+	    LOCK_ACCESS (for_write), app, store_name, buf, node_id));
+      
       return response;
     }
-  else return gzochid_data_response_new (app, store_name, FALSE, NULL);
+  else
+    {
+      if (gzochid_log_level_visible (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG))
+	GZOCHID_WITH_FORMATTED_BYTES
+	  (key, buf, 33, g_debug ("Denied %s lock on key %s/%s/%s to node %d.",
+				  LOCK_ACCESS (for_write), app, store_name, buf,
+				  node_id));
+      
+      return gzochid_data_response_new (app, store_name, FALSE, NULL);
+    }
 }
 
 gzochid_data_response *
 gzochi_metad_dataserver_request_next_key (GzochiMetadDataServer *server,
-					  guint node_id, char *app,
-					  char *store_name, GBytes *key,
+					  guint node_id, const char *app,
+					  const char *store_name, GBytes *key,
 					  GError **err)
 {
   gzochid_data_response *response = NULL;
@@ -436,11 +491,24 @@ gzochi_metad_dataserver_request_next_key (GzochiMetadDataServer *server,
     (app_store->storage_context);
   
   if (key == NULL)
-    data = STORAGE_INTERFACE (server)->transaction_first_key
-      (transaction, store->store, &data_len);
-  else data = STORAGE_INTERFACE (server)->transaction_next_key
-	 (transaction, store->store, (char *) g_bytes_get_data (key, NULL),
-	  g_bytes_get_size (key), &data_len);
+    {
+      gzochid_trace ("Node %d requested range lock on %s/%s keyspace.", node_id,
+		     app, store_name);
+      data = STORAGE_INTERFACE (server)->transaction_first_key
+	(transaction, store->store, &data_len);
+    }
+  else
+    {
+      if (gzochid_log_level_visible (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
+	GZOCHID_WITH_FORMATTED_BYTES
+	  (key, buf, 33, gzochid_trace
+	   ("Node %d requested range lock from key %s/%s/%s.", node_id, app,
+	    store_name, buf));
+	   
+      data = STORAGE_INTERFACE (server)->transaction_next_key
+	(transaction, store->store, (char *) g_bytes_get_data (key, NULL),
+	 g_bytes_get_size (key), &data_len);
+    }
   
   assert (!transaction->rollback);
   
@@ -452,8 +520,32 @@ gzochi_metad_dataserver_request_next_key (GzochiMetadDataServer *server,
 
   if (!gzochid_lock_range_check_and_set
       (store->locks, node_id, key, to_key, &most_recent_lock))
-    response = gzochid_data_response_new (app, store_name, FALSE, NULL);
-  else response = gzochid_data_response_new (app, store_name, TRUE, to_key);
+    {
+      if (key == NULL)
+	g_debug ("Denied range lock on %s/%s keyspace to node %d.", app,
+		 store_name, node_id);
+      else if (gzochid_log_level_visible (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG))
+	GZOCHID_WITH_FORMATTED_BYTES
+	  (key, buf, 33, g_debug
+	   ("Denied range lock starting at %s/%s/%s to node %d.", app,
+	    store_name, buf, node_id));
+      
+      response = gzochid_data_response_new (app, store_name, FALSE, NULL);
+    }
+  else
+    {
+      if (key == NULL)
+	gzochid_trace ("Granted range lock on %s/%s keyspace to node %d.", app,
+		       store_name, node_id);
+      else if (gzochid_log_level_visible
+	       (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
+	GZOCHID_WITH_FORMATTED_BYTES
+	  (key, buf, 33, gzochid_trace
+	   ("Granted range lock starting at %s/%s/%s to node %d.", app,
+	    store_name, buf, node_id));
+      
+      response = gzochid_data_response_new (app, store_name, TRUE, to_key);
+    }
 
   if (to_key != NULL)
     g_bytes_unref (to_key);
@@ -474,6 +566,9 @@ gzochi_metad_dataserver_process_changeset (GzochiMetadDataServer *server,
   gzochid_storage_transaction *transaction =
     STORAGE_INTERFACE (server)->transaction_begin (app_store->storage_context);
 
+  gzochid_trace ("Processing %d changes from %d/%s.", changeset->changes->len,
+		 node_id, changeset->app);
+  
   for (; i < changeset->changes->len; i++)
     {
       gzochid_data_change change = g_array_index
@@ -489,8 +584,7 @@ gzochi_metad_dataserver_process_changeset (GzochiMetadDataServer *server,
 	  break;
 	}
 
-      if (!gzochid_lock_check_and_set
-	  (store->locks, node_id, change.key, TRUE, NULL))
+      if (!gzochid_lock_check (store->locks, node_id, change.key, TRUE))
 	{
 	  g_set_error
 	    (err, GZOCHI_METAD_DATASERVER_ERROR,
@@ -529,21 +623,38 @@ gzochi_metad_dataserver_process_changeset (GzochiMetadDataServer *server,
     }
     
   if (needs_rollback)
-    STORAGE_INTERFACE (server)->transaction_rollback (transaction);
+    {
+      g_debug ("Changes from %d/%s rolled back during processing.", node_id,
+	       changeset->app);
+      STORAGE_INTERFACE (server)->transaction_rollback (transaction);
+    }
   else
     {
       STORAGE_INTERFACE (server)->transaction_prepare (transaction);
 
       if (transaction->rollback)
-	STORAGE_INTERFACE (server)->transaction_rollback (transaction);
-      else STORAGE_INTERFACE (server)->transaction_commit (transaction);
+	{
+	  g_set_error
+	    (err, GZOCHI_METAD_DATASERVER_ERROR,
+	     GZOCHI_METAD_DATASERVER_ERROR_FAILED,
+	     "Transaction failed to prepare.");
+	  g_debug ("Changes from %d/%s rolled back during prepare.", node_id,
+		   changeset->app);
+	  STORAGE_INTERFACE (server)->transaction_rollback (transaction);
+	}
+      else
+	{
+	  gzochid_trace ("Committed changes from %d/%s.", node_id,
+			 changeset->app);
+	  STORAGE_INTERFACE (server)->transaction_commit (transaction);
+	}
     }
 }
 
 void
 gzochi_metad_dataserver_release_key (GzochiMetadDataServer *server,
-				     guint node_id, char *app, char *store_name,
-				     GBytes *key)
+				     guint node_id, const char *app,
+				     const char *store_name, GBytes *key)
 {
   gzochi_metad_dataserver_application_store *app_store =
     ensure_open_application_store (server, app);
@@ -558,16 +669,22 @@ gzochi_metad_dataserver_release_key (GzochiMetadDataServer *server,
 	("Failed to release key for application '%s': %s", app,
 	 local_err->message);
       g_error_free (local_err);
+      return;
     }
+
+  if (gzochid_log_level_visible (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
+    GZOCHID_WITH_FORMATTED_BYTES
+      (key, buf, 33, gzochid_trace ("Node id %d releasing key %s/%s/%s.",
+				    node_id, app, store_name, buf));
   
-  gzochid_lock_release (store->locks, node_id, key);  
+  gzochid_lock_release (store->locks, node_id, key);
 }
 
 void
 gzochi_metad_dataserver_release_range (GzochiMetadDataServer *server,
-				       guint node_id, char *app,
-				       char *store_name, GBytes *first_key,
-				       GBytes *last_key)
+				       guint node_id, const char *app,
+				       const char *store_name,
+				       GBytes *first_key, GBytes *last_key)
 {
   gzochi_metad_dataserver_application_store *app_store =
     ensure_open_application_store (server, app);
@@ -579,11 +696,21 @@ gzochi_metad_dataserver_release_range (GzochiMetadDataServer *server,
   if (local_err != NULL)
     {
       g_warning
-	("Failed to release key for application '%s': %s", app,
+	("Failed to release key range for application '%s': %s", app,
 	 local_err->message);
       g_error_free (local_err);
+      return;
     }
 
+  if (first_key == NULL)
+    gzochid_trace ("Node %d releasing range lock on %s/%s keyspace.", node_id,
+		   app, store_name);
+  else if (gzochid_log_level_visible (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
+    GZOCHID_WITH_FORMATTED_BYTES
+      (first_key, buf, 33, gzochid_trace 
+       ("Node id %d releasing range lock from %s/%s/%s.", node_id, app,
+	store_name, buf));
+  
   gzochid_lock_release_range (store->locks, node_id, first_key, last_key);
 }
 
