@@ -1,5 +1,5 @@
 /* gzochi-metad.c: Main server bootstrapping routines for gzochi-metad
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -26,12 +26,16 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "channelserver.h"
 #include "config.h"
 #include "dataserver.h"
-#include "httpd.h"
+#include "event.h"
 #include "httpd-meta.h"
+#include "httpd.h"
 #include "log.h"
+#include "metaserver-protocol.h"
 #include "resolver.h"
+#include "sessionserver.h"
 #include "socket.h"
 
 #define _(String) gettext (String)
@@ -63,15 +67,44 @@ typedef struct _GzochiMetadRootContextClass GzochiMetadRootContextClass;
 
 struct _GzochiMetadRootContext
 {
-  GObject parent_intance;
+  GObject parent_instance;
 
+  GzochidConfiguration *configuration; /* The global configuration. */  
   GzochidHttpServer *http_server; /* The admin web console. */
 
+  /* When the data server has been started, holds the base URL of the gzochid 
+     admin HTTP console, if available; otherwise, a heap-allocated empty 
+     string. */
+
+  char *admin_server_base_url;
+  
+  GzochidEventLoop *event_loop; /* The global event loop. */
+
+  /* An event source for client connect / disconnect events. */
+
+  gzochid_event_source *event_source; 
+
   /* The meta server global socket server. */
 
   GzochidSocketServer *socket_server;
+
+  gzochid_server_socket *server_socket; /* The meta server's server socket. */
+
+  /* The meta server channel server. */
+  
+  GzochiMetadChannelServer *channel_server;
   
   GzochiMetadDataServer *data_server; /* The meta server data server. */
+
+  /* The meta server session server. */
+
+  GzochiMetadSessionServer *session_server; 
+
+  /* The port on which the server listens. This may be zero to indicate the 
+     server should listen on any available port, and will be set to the port 
+     assigned by the operating system. */
+  
+  int port;
 };
 
 typedef struct _GzochiMetadRootContext GzochiMetadRootContext;
@@ -87,14 +120,54 @@ G_DEFINE_TYPE (GzochiMetadRootContext, gzochi_metad_root_context,
 
 enum gzochi_metad_root_context_properties
   {
-    PROP_HTTP_SERVER = 1,
+    PROP_CONFIGURATION = 1,
+    PROP_HTTP_SERVER,
+    PROP_ADMIN_SERVER_BASE_URL,
+    PROP_EVENT_LOOP,
+    PROP_EVENT_SOURCE,
     PROP_SOCKET_SERVER,
+    PROP_CHANNEL_SERVER,
     PROP_DATA_SERVER,
+    PROP_SESSION_SERVER,
     N_PROPERTIES
   };
 
 static GParamSpec *obj_properties[N_PROPERTIES] = { NULL };
 
+static void
+root_context_get_property (GObject *object, guint property_id, GValue *value,
+			   GParamSpec *pspec)
+{
+  GzochiMetadRootContext *self = GZOCHI_METAD_ROOT_CONTEXT (object);
+
+  switch (property_id)
+    {
+    case PROP_ADMIN_SERVER_BASE_URL:
+      g_value_set_static_string (value, self->admin_server_base_url);
+      break;
+
+    case PROP_EVENT_SOURCE:
+      g_value_set_boxed (value, self->event_source);
+      break;
+
+    case PROP_CHANNEL_SERVER:
+      g_value_set_object (value, self->channel_server);
+      break;
+      
+    case PROP_DATA_SERVER:
+      g_value_set_object (value, self->data_server);
+      break;
+
+    case PROP_SESSION_SERVER:
+      g_value_set_object (value, self->session_server);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+    }
+}
+
 static void
 root_context_set_property (GObject *object, guint property_id,
 			   const GValue *value, GParamSpec *pspec)
@@ -103,44 +176,125 @@ root_context_set_property (GObject *object, guint property_id,
 
   switch (property_id)
     {
+    case PROP_CONFIGURATION:
+      self->configuration = g_object_ref (g_value_get_object (value));
+      break;
+      
     case PROP_HTTP_SERVER:
       self->http_server = g_object_ref (g_value_get_object (value));
       break;
 
+    case PROP_EVENT_LOOP:
+      self->event_loop = g_object_ref (g_value_get_object (value));
+      break;
+      
     case PROP_SOCKET_SERVER:
       self->socket_server = g_object_ref (g_value_get_object (value));
       break;
 
+    case PROP_CHANNEL_SERVER:
+      self->channel_server = g_object_ref (g_value_get_object (value));
+      break;
+      
     case PROP_DATA_SERVER:
       self->data_server = g_object_ref (g_value_get_object (value));
       break;
-      
+
+    case PROP_SESSION_SERVER:
+      self->session_server = g_object_ref (g_value_get_object (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
     }
 }
 
+static void
+root_context_constructed (GObject *object)
+{
+  GzochiMetadRootContext *root_context = GZOCHI_METAD_ROOT_CONTEXT (object);
+  GHashTable *meta_configuration = gzochid_configuration_extract_group
+    (root_context->configuration, "meta");
+    
+  root_context->port = gzochid_config_to_int
+    (g_hash_table_lookup (meta_configuration, "server.port"), 9001);
+
+  g_hash_table_destroy (meta_configuration);
+
+  gzochid_event_source_attach
+    (root_context->event_loop, root_context->event_source);
+}
+
+static void
+root_context_dispose (GObject *object)
+{
+  GzochiMetadRootContext *root_context = GZOCHI_METAD_ROOT_CONTEXT (object);
+
+  g_object_unref (root_context->configuration);
+
+  G_OBJECT_CLASS (gzochi_metad_root_context_parent_class)->dispose (object);
+}
+
+static void
+root_context_finalize (GObject *object)
+{
+  GzochiMetadRootContext *root_context = GZOCHI_METAD_ROOT_CONTEXT (object);
+  
+  g_source_destroy ((GSource *) root_context->event_source);
+  g_source_unref ((GSource *) root_context->event_source);
+
+  G_OBJECT_CLASS (gzochi_metad_root_context_parent_class)->finalize (object);
+}
+
 static void
 gzochi_metad_root_context_class_init (GzochiMetadRootContextClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->constructed = root_context_constructed;
+  object_class->dispose = root_context_dispose;
+  object_class->finalize = root_context_finalize;
+  object_class->get_property = root_context_get_property;
   object_class->set_property = root_context_set_property;
+
+  obj_properties[PROP_CONFIGURATION] = g_param_spec_object
+    ("configuration", "config", "The global configuration object",
+     GZOCHID_TYPE_CONFIGURATION, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+  
   obj_properties[PROP_HTTP_SERVER] = g_param_spec_object
-    ("http-server", "Admin HTTP server", "Set the admin HTTP server",
-     GZOCHID_TYPE_HTTP_SERVER,
-     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
+    ("http-server", "httpd", "The admin HTTP server", GZOCHID_TYPE_HTTP_SERVER,
+     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+
+  obj_properties[PROP_ADMIN_SERVER_BASE_URL] = g_param_spec_string
+    ("admin-server-base-url", "base-url", "The admin server base URL", NULL,
+     G_PARAM_READABLE);
+  
+  obj_properties[PROP_EVENT_LOOP] = g_param_spec_object
+    ("event-loop", "event-loop", "The global event loop",
+     GZOCHID_TYPE_EVENT_LOOP, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+  
+  obj_properties[PROP_EVENT_SOURCE] = g_param_spec_boxed
+    ("event-source", "event-source", "The global event source",
+     G_TYPE_SOURCE, G_PARAM_READABLE);
 
   obj_properties[PROP_SOCKET_SERVER] = g_param_spec_object
-    ("socket-server", "Socket server", "Set the socket server",
-     GZOCHID_TYPE_SOCKET_SERVER,
-     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
+    ("socket-server", "socket-server", "The global socket server",
+     GZOCHID_TYPE_SOCKET_SERVER, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+
+  obj_properties[PROP_CHANNEL_SERVER] = g_param_spec_object
+    ("channel-server", "channel-server", "The channel server",
+     GZOCHI_METAD_TYPE_CHANNEL_SERVER,
+     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
   obj_properties[PROP_DATA_SERVER] = g_param_spec_object
-    ("data-server", "Data server", "Set the data server",
-     GZOCHI_METAD_TYPE_DATA_SERVER,
-     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
+    ("data-server", "data-server", "The data server",
+     GZOCHI_METAD_TYPE_DATA_SERVER, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
+  obj_properties[PROP_SESSION_SERVER] = g_param_spec_object
+    ("session-server", "session-server", "The session server",
+     GZOCHI_METAD_TYPE_SESSION_SERVER,
+     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
   g_object_class_install_properties
     (object_class, N_PROPERTIES, obj_properties);
@@ -149,6 +303,10 @@ gzochi_metad_root_context_class_init (GzochiMetadRootContextClass *klass)
 static void
 gzochi_metad_root_context_init (GzochiMetadRootContext *self)
 {
+  self->event_source = gzochid_event_source_new ();
+  self->server_socket = gzochid_server_socket_new
+    ("Meta server", gzochi_metad_metaserver_server_protocol, self);
+  self->port = 0;
 }
 
 static const struct option longopts[] =
@@ -172,7 +330,7 @@ Copyright (C) %s Julian Graham\n\
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n"),
-	  "2016");
+	  "2017");
 }
 
 /* Prints help and configuration options to standard output. */
@@ -242,7 +400,9 @@ initialize_logging (GKeyFile *key_file)
 /* Configure and start the admin web console if it's enabled. */
 
 static void
-initialize_httpd (GzochidHttpServer *http_server, GKeyFile *key_file)
+initialize_httpd (GzochidHttpServer *http_server,
+		  gzochid_event_source *root_event_source,
+		  GzochidResolutionContext *res_context, GKeyFile *key_file)
 {
   GHashTable *admin_config =
     gzochid_config_keyfile_extract_config (key_file, "admin");
@@ -250,11 +410,20 @@ initialize_httpd (GzochidHttpServer *http_server, GKeyFile *key_file)
   if (gzochid_config_to_boolean
       (g_hash_table_lookup (admin_config, "module.httpd.enabled"), FALSE))
     {
+      GError *err = NULL;
       int port = gzochid_config_to_int
 	(g_hash_table_lookup (admin_config, "module.httpd.port"), 8800);
 
-      gzochid_httpd_meta_register_handlers (http_server);
-      gzochid_http_server_start (http_server, port, NULL);
+      gzochid_httpd_meta_register_handlers
+	(http_server, root_event_source, res_context);
+      gzochid_http_server_start (http_server, port, &err);
+
+      if (err != NULL)
+	{
+	  g_critical
+	    ("Failed to start admin HTTP server: %s; exiting...", err->message);
+	  exit (EXIT_FAILURE);
+	}
     }
   
   g_hash_table_destroy (admin_config);
@@ -325,7 +494,7 @@ main (int argc, char *argv[])
 
   resolution_context = g_object_new (GZOCHID_TYPE_RESOLUTION_CONTEXT, NULL);
   configuration = g_object_new
-    (GZOCHID_TYPE_CONFIGURATION, "key_file", key_file, NULL);
+    (GZOCHID_TYPE_CONFIGURATION, "key_file", key_file, "path", conf_path, NULL);
   gzochid_resolver_provide (resolution_context, G_OBJECT (configuration), NULL);
   
   root_context = gzochid_resolver_require_full
@@ -337,8 +506,21 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  initialize_httpd (root_context->http_server, key_file);
+  gzochid_event_loop_start (root_context->event_loop);
+  initialize_httpd (root_context->http_server, root_context->event_source,
+		    resolution_context, key_file);
+
+  if (root_context->http_server != NULL)
+    root_context->admin_server_base_url =
+      strdup (gzochid_http_server_get_base_url (root_context->http_server));
+  else root_context->admin_server_base_url = strdup ("");
+
   gzochi_metad_dataserver_start (root_context->data_server);
+  
+  gzochid_server_socket_listen
+    (root_context->socket_server, root_context->server_socket,
+     root_context->port);
+  
   g_main_loop_run (root_context->socket_server->main_loop);
   
   return 0;
