@@ -1,5 +1,5 @@
 /* httpd-app.c: Handler implementations for the gzochid admin web console
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -24,11 +24,17 @@
 #include <string.h>
 
 #include "app.h"
+#include "config.h"
+#include "event.h"
+#include "event-app.h"
 #include "game.h"
 #include "httpd.h"
 #include "httpd-app.h"
+#include "resolver.h"
 #include "util.h"
 
+#define HEADER "  <head><title>gzochid v" VERSION "</title></head>"
+
 /* The following declaraitons support rendering the contents of an object in the
    object store. */
 
@@ -36,6 +42,89 @@
 #define OID_SUFFIX_LEN 29
 #define OID_LINE_LEN 80
 
+/* Holds information about the current meta server connection. */
+
+struct _gzochid_metaserver_info
+{
+  char *connection_description; /* The socket address of the meta server. */
+
+  /* The base URL of the meta server's admin HTTP console, or `NULL' if the
+     meta server didn't provide an address. */
+
+  char *admin_server_base_url; 
+};
+
+typedef struct _gzochid_metaserver_info gzochid_metaserver_info;
+
+/* Construct and return new `gzochid_metaserver_info' instance, copying the
+   specified connection description and admin server base URL (which can be 
+   `NULL'). The returned pointer should be freed via 
+   `gzochid_metaserver_info_free' when no longer in use. */
+
+static gzochid_metaserver_info *
+gzochid_metaserver_info_new (const char *connection_description,
+			     const char *admin_server_base_url)
+{
+  gzochid_metaserver_info *info = malloc (sizeof (gzochid_metaserver_info));
+
+  info->connection_description = strdup (connection_description);
+
+  if (admin_server_base_url != NULL)
+    info->admin_server_base_url = strdup (admin_server_base_url);
+  else info->admin_server_base_url = NULL;
+  
+  return info;
+}
+
+/* Release the memory associated with the specified `gzochid_metaserver_info'
+   object. */
+
+static void
+gzochid_metaserver_info_free (gzochid_metaserver_info *info)
+{
+  free (info->connection_description);
+
+  if (info->admin_server_base_url != NULL)
+    free (info->admin_server_base_url);
+
+  free (info);
+}
+
+/* The state of the gzochid application container, in terms of the information
+   displayed on the admin console. This is effectively an "offline"
+   representation, to decouple the rendering of the admin console from what's
+   actually going on in the container. */
+
+struct _gzochid_server_state
+{
+  /* `TRUE' if the server has a metaclient configured, `FALSE' otherwise. */
+
+  gboolean has_metaclient; 
+  
+  /* Information about the currently-connected meta server, if such a connection
+     has been configured and is active; otherwise, `NULL'. */
+  
+  gzochid_metaserver_info *metaserver_info;
+
+  GMutex mutex; /* Mutex to protect updates to the state object. */
+};
+
+typedef struct _gzochid_server_state gzochid_server_state;
+
+/* Construct and return a pointer to a new `gzochid_server_state' object. */
+
+static gzochid_server_state *
+gzochid_server_state_new ()
+{
+  gzochid_server_state *state = malloc (sizeof (gzochid_server_state));
+
+  state->has_metaclient = FALSE;
+  state->metaserver_info = NULL;
+  g_mutex_init (&state->mutex);
+  
+  return state;
+}
+
 /* Captures serialization state of an object. */
 
 struct data_state 
@@ -192,16 +281,57 @@ not_found404_default (gzochid_http_response_sink *sink)
   "applications running within container, including their attached data " \
   "stores.</p>"
 
+/* Writes a global header to the specified outbut buffer. */
+
+static void
+append_header (GString *response_str)
+{
+  g_string_append (response_str, "<html>\n");
+  g_string_append (response_str, HEADER);
+  g_string_append (response_str, "  <body>\n");
+}
+
+/* Writes a global footer to the specified output buffer. */
+
+static void
+append_footer (GString *response_str)
+{
+  g_string_append (response_str, "  </body>\n");
+  g_string_append (response_str, "</html>\n");
+}
+
 static void
 hello_world (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 	     gpointer request_context, gpointer user_data)
 {
+  gzochid_server_state *state = user_data;
   GString *response_str = g_string_new (NULL);
 
-  g_string_append (response_str, "<html><body>");
+  append_header (response_str);
   g_string_append (response_str, GREETING);
+
+  g_mutex_lock (&state->mutex);
+
+  if (state->metaserver_info != NULL)
+    {
+      g_string_append (response_str, "<p>Connected to the meta server at ");
+
+      if (state->metaserver_info->admin_server_base_url != NULL)
+	g_string_append_printf
+	  (response_str, "<a href=\"%s\">%s</a>",
+	   state->metaserver_info->admin_server_base_url,
+	   state->metaserver_info->connection_description);
+
+      else g_string_append
+	     (response_str, state->metaserver_info->connection_description);
+      
+      g_string_append (response_str, ".</p>\n");
+    }
+  
+  g_mutex_unlock (&state->mutex);
+  
   g_string_append (response_str, "<a href=\"app/\">app/</a>");
-  g_string_append (response_str, "</body></html>");
+  append_footer (response_str);
 
   gzochid_http_write_response
     (sink, 200, response_str->str, response_str->len);
@@ -213,8 +343,8 @@ static void
 list_apps (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 	   gpointer request_context, gpointer user_data)
 {
-  gzochid_game_context *game_context = user_data;
-  GList *apps = gzochid_game_context_get_applications (game_context);
+  GzochidGameServer *game_server = user_data;
+  GList *apps = gzochid_game_server_get_applications (game_server);
 
   if (apps == NULL)
     gzochid_http_write_response
@@ -224,8 +354,7 @@ list_apps (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
       GList *apps_ptr = apps;
       GString *response_str = g_string_new (NULL);
 
-      g_string_append (response_str, "<html>\n");
-      g_string_append (response_str, "  <body>\n");
+      append_header (response_str);
       g_string_append (response_str, "    <ul>\n");
       
       while (apps_ptr != NULL)
@@ -241,8 +370,7 @@ list_apps (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 	}
 
       g_string_append (response_str, "    </ul>\n");
-      g_string_append (response_str, "  </body>\n");
-      g_string_append (response_str, "</html>");
+      append_footer (response_str);
 
       gzochid_http_write_response
 	(sink, 200, response_str->str, response_str->len);
@@ -257,9 +385,9 @@ bind_app (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 	  gpointer request_context, gpointer user_data)
 {
   gchar *app = g_match_info_fetch (match_info, 1);
-  gzochid_game_context *game_context = user_data;
+  GzochidGameServer *game_server = user_data;
   gzochid_application_context *app_context =
-    gzochid_game_context_lookup_application (game_context, app);
+    gzochid_game_server_lookup_application (game_server, app);
 
   g_free (app);
   
@@ -275,22 +403,29 @@ app_info (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 {
   GString *response_str = g_string_new (NULL);
   gzochid_application_context *app_context = request_context;
+  gzochid_server_state *state = user_data;
 
-  g_string_append (response_str, "<html>\n");
-  g_string_append (response_str, "  <body>\n");
+  append_header (response_str);
 
   g_string_append_printf 
     (response_str, "    <h1>%s</h1>\n", app_context->descriptor->name);
   g_string_append_printf 
     (response_str, "    <p>%s</p>\n", app_context->descriptor->description);
-  g_string_append (response_str, "    <h2>Application data</h2>\n");
-  g_string_append_printf 
-    (response_str, "    <a href=\"/app/%s/names/\">names</a><br />\n", 
-     app_context->descriptor->name);
-  g_string_append_printf 
-    (response_str, "    <a href=\"/app/%s/oids/\">oids</a><br />\n", 
-     app_context->descriptor->name);
 
+  /* Shouldn't show users links to dump contents of store if we're in 
+     distributed mode. */
+  
+  if (!state->has_metaclient)
+    {
+      g_string_append (response_str, "    <h2>Application data</h2>\n");
+      g_string_append_printf 
+	(response_str, "    <a href=\"/app/%s/names/\">names</a><br />\n", 
+	 app_context->descriptor->name);
+      g_string_append_printf 
+	(response_str, "    <a href=\"/app/%s/oids/\">oids</a><br />\n", 
+	 app_context->descriptor->name);
+    }
+  
   g_string_append (response_str, "    <h2>Application statistics</h2>\n");
   g_string_append (response_str, "    <table>\n");
 
@@ -361,8 +496,7 @@ app_info (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
     }
 
   g_string_append (response_str, "    </table>\n");
-  g_string_append (response_str, "  </body>\n");
-  g_string_append (response_str, "</html>");
+  append_footer (response_str);
 
   gzochid_http_write_response
     (sink, 200, response_str->str, response_str->len);
@@ -378,17 +512,16 @@ list_oids (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
   GString *response_str = g_string_new (NULL);
   
   size_t klen = 0;
-  gzochid_storage_engine_interface *iface = APP_STORAGE_INTERFACE (app_context);
-  gzochid_storage_transaction *tx = iface->transaction_begin
-    (app_context->storage_context);
-  char *k = iface->transaction_first_key (tx, app_context->oids, &klen);
+  gzochid_storage_transaction *tx = app_context->storage_engine_interface
+    ->transaction_begin (app_context->storage_context);
+  char *k = app_context->storage_engine_interface
+    ->transaction_first_key (tx, app_context->oids, &klen);
 
-  iface->transaction_rollback (tx);
+  app_context->storage_engine_interface->transaction_rollback (tx);
 
   assert (klen == sizeof (guint64));  
 
-  g_string_append (response_str, "<html>\n");
-  g_string_append (response_str, "  <body>\n");
+  append_header (response_str);
   g_string_append_printf 
     (response_str, "    <h1>%s</h1><br />\n", app_context->descriptor->name);
   g_string_append (response_str, "    <ul>\n");
@@ -410,18 +543,18 @@ list_oids (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 	 gzochid_util_decode_oid (encoded_oid));
       g_string_append (response_str, "</a></li>\n");
 
-      tx = iface->transaction_begin (app_context->storage_context);      
-      next_k = iface->transaction_next_key
+      tx = app_context->storage_engine_interface
+	->transaction_begin (app_context->storage_context);      
+      next_k = app_context->storage_engine_interface->transaction_next_key
 	(tx, app_context->oids, k, klen, &klen);
-      iface->transaction_rollback (tx);
+      app_context->storage_engine_interface->transaction_rollback (tx);
       
       free (k);
       k = next_k;
     }
   
   g_string_append (response_str, "    </ul>\n");
-  g_string_append (response_str, "  </body>\n");
-  g_string_append (response_str, "</html>");
+  append_footer (response_str);
 
   gzochid_http_write_response
     (sink, 200, response_str->str, response_str->len);
@@ -436,14 +569,13 @@ render_oid (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
   size_t data_length = 0;
   char *oid_str = g_match_info_fetch (match_info, 1);
   gzochid_application_context *app_context = request_context;
-  gzochid_storage_engine_interface *iface = APP_STORAGE_INTERFACE (app_context);
-  gzochid_storage_transaction *tx = iface->transaction_begin
-    (app_context->storage_context);
+  gzochid_storage_transaction *tx = app_context->storage_engine_interface
+    ->transaction_begin (app_context->storage_context);
   guint64 oid = gzochid_util_encode_oid (g_ascii_strtoull (oid_str, NULL, 16)); 
-  char *data = iface->transaction_get
+  char *data = app_context->storage_engine_interface->transaction_get
     (tx, app_context->oids, (char *) &oid, sizeof (guint64), &data_length);
   
-  iface->transaction_rollback (tx);
+  app_context->storage_engine_interface->transaction_rollback (tx);
   
   free (oid_str);
 
@@ -476,15 +608,14 @@ list_names (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
   
   size_t klen = 0;
   gzochid_application_context *app_context = request_context;
-  gzochid_storage_engine_interface *iface = APP_STORAGE_INTERFACE (app_context);
-  gzochid_storage_transaction *tx = iface->transaction_begin
-    (app_context->storage_context);
-  char *k = iface->transaction_first_key (tx, app_context->names, &klen);
+  gzochid_storage_transaction *tx = app_context->storage_engine_interface
+    ->transaction_begin (app_context->storage_context);
+  char *k = app_context->storage_engine_interface
+    ->transaction_first_key (tx, app_context->names, &klen);
 
-  iface->transaction_rollback (tx);
-  
-  g_string_append (response_str, "<html>\n");
-  g_string_append (response_str, "  <body>\n");
+  app_context->storage_engine_interface->transaction_rollback (tx);
+
+  append_header (response_str);
   g_string_append_printf 
     (response_str, "    <h1>%s</h1><br />\n", app_context->descriptor->name);
   g_string_append (response_str, "    <ul>\n");
@@ -497,18 +628,18 @@ list_names (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
       g_string_append_len (response_str, k, klen);
       g_string_append (response_str, "</li>\n");
 
-      tx = iface->transaction_begin (app_context->storage_context);      
-      next_k = iface->transaction_next_key
+      tx = app_context->storage_engine_interface
+	->transaction_begin (app_context->storage_context);      
+      next_k = app_context->storage_engine_interface->transaction_next_key
 	(tx, app_context->names, k, klen, &klen);
-      iface->transaction_rollback (tx);
+      app_context->storage_engine_interface->transaction_rollback (tx);
       
       free (k);
       k = next_k;
     }
 
   g_string_append (response_str, "    </ul>\n");
-  g_string_append (response_str, "  </body>\n");
-  g_string_append (response_str, "</html>");
+  append_footer (response_str);
 
   gzochid_http_write_response
     (sink, 200, response_str->str, response_str->len);
@@ -527,25 +658,125 @@ null_continuation (const GMatchInfo *match_info,
   return request_context;
 }
 
+/* `gzochid_event_handler' implementation to update the admin console's
+   representation of the server state in response to meta server events. */
+
+static void
+handle_metaserver_event (GzochidEvent *event, gpointer user_data)
+{
+  gzochid_server_state *state = user_data;
+  GzochidMetaServerEvent *metaserver_event = GZOCHID_META_SERVER_EVENT (event);
+  gzochid_meta_server_event_type type;
+
+  g_object_get (metaserver_event, "type", &type, NULL);
+
+  g_mutex_lock (&state->mutex);
+
+  switch (type)
+    {
+    case META_SERVER_CONNECTED:
+      {
+	char *connection_description = NULL;
+	char *admin_server_base_url = NULL;
+
+	g_object_get
+	  (metaserver_event,
+	   "connection-description", &connection_description,
+	   "admin-server-base-url", &admin_server_base_url,
+	   NULL);
+
+	state->metaserver_info = gzochid_metaserver_info_new
+	  (connection_description, admin_server_base_url);
+
+	g_free (connection_description);
+
+	if (admin_server_base_url != NULL)
+	  g_free (admin_server_base_url);
+	break;
+      }
+      
+    case META_SERVER_DISCONNECTED:
+
+      if (state->metaserver_info != NULL)
+	{
+	  gzochid_metaserver_info_free (state->metaserver_info);
+	  state->metaserver_info = NULL;
+	}
+      break;
+    }
+  
+  g_mutex_unlock (&state->mutex);
+}
+
+/* Conditionally attach the admin console's handler for meta server events to
+   the data client's event source, if a data client is available. Also set the
+   `has_metaclient' flag on the server state. */
+
+static void
+attach_data_client_handler (GzochidResolutionContext *res_context,
+			    gzochid_server_state *state)
+{
+  GzochidConfiguration *configuration = gzochid_resolver_require_full
+    (res_context, GZOCHID_TYPE_CONFIGURATION, NULL);
+  GzochidMetaClientContainer *container = gzochid_resolver_require_full
+    (res_context, GZOCHID_TYPE_META_CLIENT_CONTAINER, NULL);
+  GzochidMetaClient *metaclient = NULL;
+
+  g_object_get (container, "metaclient", &metaclient, NULL);
+  
+  if (metaclient != NULL)
+    {
+      /* If there's a meta server configured, a data client object should be
+	 available. Require it and grab its event source to attach the handler
+	 for meta server events. */
+      
+      gzochid_event_source *event_source = NULL;
+
+      g_object_get (metaclient, "event-source", &event_source, NULL);
+      gzochid_event_attach (event_source, handle_metaserver_event, state);
+      g_source_unref ((GSource *) event_source);
+
+      g_object_unref (metaclient);
+
+      state->has_metaclient = TRUE;
+    }
+
+  g_object_unref (container);
+  g_object_unref (configuration);
+}
+
 void
 gzochid_httpd_app_register_handlers (GzochidHttpServer *http_server,
-				     gzochid_game_context *game_context)
+				     GzochidGameServer *game_server,
+				     GzochidResolutionContext *res_context)
 {
   gzochid_httpd_partial *apps_root = NULL;
   gzochid_httpd_partial *oids_root = NULL;
+
+  gzochid_server_state *state = gzochid_server_state_new ();
+
+  attach_data_client_handler (res_context, state);
   
-  gzochid_httpd_add_terminal (http_server, "/", hello_world, NULL);
-  gzochid_httpd_add_terminal (http_server, "/app/", list_apps, game_context);
+  gzochid_httpd_add_terminal (http_server, "/", hello_world, state);
+  gzochid_httpd_add_terminal (http_server, "/app/", list_apps, game_server);
 
   apps_root = gzochid_httpd_add_continuation
-    (http_server, "/app/([^/]+)", bind_app, game_context);
+    (http_server, "/app/([^/]+)", bind_app, game_server);
 
-  gzochid_httpd_append_terminal (apps_root, "/?", app_info, NULL);
-  gzochid_httpd_append_terminal (apps_root, "/oids/", list_oids, NULL);
-  oids_root = gzochid_httpd_append_continuation
-    (apps_root, "/oids/", null_continuation, NULL);
+  gzochid_httpd_append_terminal (apps_root, "/?", app_info, state);
+
+  /* Don't register handlers for dumping contents of the store if the server's
+     running in distribured mode. */
   
-  gzochid_httpd_append_terminal (oids_root, "([a-f0-9]+)", render_oid, NULL);
+  if (!state->has_metaclient)
+    {
+      gzochid_httpd_append_terminal (apps_root, "/oids/", list_oids, NULL);
+      oids_root = gzochid_httpd_append_continuation
+	(apps_root, "/oids/", null_continuation, NULL);
+  
+      gzochid_httpd_append_terminal
+	(oids_root, "([a-f0-9]+)", render_oid, NULL);
 
-  gzochid_httpd_append_terminal (apps_root, "/names/", list_names, NULL);
+      gzochid_httpd_append_terminal (apps_root, "/names/", list_names, NULL);
+    }
 }
