@@ -1,5 +1,5 @@
 /* session.c: Client session management routines for gzochid
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 #include <assert.h>
 #include <glib.h>
+#include <glib-object.h>
 #include <gmp.h>
 #include <stddef.h>
 #include <stdlib.h>
@@ -26,13 +27,14 @@
 #include "auth_int.h"
 #include "data.h"
 #include "event.h"
-#include "game.h"
 #include "game-protocol.h"
 #include "gzochid-auth.h"
 #include "io.h"
+#include "metaclient.h"
 #include "reloc.h"
 #include "scheme-task.h"
 #include "session.h"
+#include "sessionclient.h"
 #include "task.h"
 #include "tx.h"
 #include "txlog.h"
@@ -187,6 +189,47 @@ cleanup_transaction (gzochid_client_session_transaction_context *tx_context)
   free (tx_context);
 }
 
+/* Relays a "disconnect" message via the meta server to the target session which
+   may live on a different gzochid application server node. */
+
+static void
+forward_disconnect (GzochidMetaClient *metaclient, char *app,
+ 		    guint64 target_session_oid)
+{
+  GzochidSessionClient *session_client = NULL;      
+  
+  g_debug ("Client not found for session '%s/%" G_GUINT64_FORMAT "'; "
+ 	   "forwarding disconnect to meta server.", app, target_session_oid);
+  
+  g_object_get (metaclient, "session-client", &session_client, NULL);
+  
+  gzochid_sessionclient_relay_disconnect_from
+    (session_client, app, target_session_oid);
+  g_object_unref (session_client);
+}
+ 
+/* Relays the specified message via the meta server for the target session which
+   may live on a different gzochid application server node. */
+
+static void
+forward_message (GzochidMetaClient *metaclient, char *app,
+ 		 guint64 target_session_oid, unsigned char *msg, short len)
+{
+  GBytes *msg_bytes = g_bytes_new (msg, len);
+  GzochidSessionClient *session_client = NULL;
+  
+  g_debug ("Client not found for session '%s/%" G_GUINT64_FORMAT "'; "
+ 	   "forwarding message to meta server.", app, target_session_oid);
+  
+  g_object_get (metaclient, "session-client", &session_client, NULL);
+  
+  gzochid_sessionclient_relay_message_from
+    (session_client, app, target_session_oid, msg_bytes);
+  
+  g_object_unref (session_client);
+  g_bytes_unref (msg_bytes);
+} 
+
 static void 
 session_commit_operation
 (gzochid_client_session_transaction_context *tx_context, 
@@ -195,11 +238,14 @@ session_commit_operation
   gzochid_application_context *context = tx_context->context;
   gzochid_client_session_pending_message_operation *msg_op = NULL;
   gzochid_game_client *client = NULL;
-
+  
   g_mutex_lock (&context->client_mapping_lock);
   client = g_hash_table_lookup (context->oids_to_clients, &op->target_session);
 
-  if (client == NULL)
+  /* If the client isn't connected locally, and this server's not part of a
+     cluster, then there's nothing to be done. */
+  
+  if (client == NULL && context->metaclient == NULL)
     {
       g_warning
 	("Client not found for session '%" G_GUINT64_FORMAT
@@ -212,7 +258,15 @@ session_commit_operation
   switch (op->type)
     {
     case GZOCHID_CLIENT_SESSION_OP_DISCONNECT:
-      gzochid_game_client_disconnect (client); break;
+
+      if (client != NULL)
+	gzochid_game_client_disconnect (client); 
+      else if (context->metaclient != NULL)
+	forward_disconnect
+	  (context->metaclient, context->descriptor->name, op->target_session);
+
+      break;
+      
     case GZOCHID_CLIENT_SESSION_OP_LOGIN_SUCCESS:
       gzochid_game_client_login_success (client); break;
     case GZOCHID_CLIENT_SESSION_OP_LOGIN_FAILURE:
@@ -223,10 +277,16 @@ session_commit_operation
 	  || op->target_session != tx_context->login_operation->target_session) 
 	{
 	  msg_op = (gzochid_client_session_pending_message_operation *) op;
-	  gzochid_application_event_dispatch
+
+	  gzochid_event_dispatch
 	    (context->event_source,
-	     gzochid_application_event_new (MESSAGE_SENT));
-	  gzochid_game_client_send (client, msg_op->message, msg_op->len);
+	     g_object_new (GZOCHID_TYPE_EVENT, "type", MESSAGE_SENT, NULL));
+
+	  if (client != NULL)
+	    gzochid_game_client_send (client, msg_op->message, msg_op->len);
+	  else if (context->metaclient != NULL)
+	    forward_message (context->metaclient, context->descriptor->name,
+			     op->target_session, msg_op->message, msg_op->len);
 	}
 
       break;
@@ -375,7 +435,7 @@ remove_session (gzochid_application_context *context, guint64 session_oid,
   GError *local_err = NULL;
   GString *binding = g_string_new (SESSION_PREFIX);
 
-  g_string_printf (binding, "%" G_GUINT64_FORMAT, session_oid);
+  g_string_append_printf (binding, "%" G_GUINT64_FORMAT, session_oid);
   gzochid_data_remove_binding (context, binding->str, &local_err);
   if (local_err == NULL)
     {
@@ -670,7 +730,7 @@ persistence_task_worker (gzochid_application_context *context,
   persistence_task->holder->oid = reference->oid;
 
   binding = g_string_new (persistence_task->prefix);
-  g_string_printf (binding, "%" G_GUINT64_FORMAT, reference->oid);
+  g_string_append_printf (binding, "%" G_GUINT64_FORMAT, reference->oid);
 
   g_clear_error (&persistence_task->holder->err);
   
@@ -730,14 +790,12 @@ gzochid_client_session_persist (gzochid_application_context *context,
 				gzochid_client_session *session, guint64 *oid,
 				GError **err)
 {
-  gzochid_game_context *game_context = 
-    (gzochid_game_context *) ((gzochid_context *) context)->parent;
   gzochid_oid_holder *holder = gzochid_oid_holder_new ();
   gzochid_task *task = gzochid_data_persistence_task_new
     (context, session->identity, &gzochid_client_session_serialization, session,
      holder, SESSION_PREFIX);
 
-  gzochid_schedule_run_task (game_context->task_queue, task);  
+  gzochid_schedule_run_task (context->task_queue, task);  
   gzochid_task_free (task);
 
   if (holder->err != NULL)
