@@ -1,5 +1,5 @@
 /* session.h: Prototypes and declarations for session.c
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 #define GZOCHID_SESSION_H
 
 #include <glib.h>
-#include <gmp.h>
 
 #include "app.h"
 #include "data.h"
@@ -46,17 +45,6 @@ struct _gzochid_client_session_handler
 
 typedef struct _gzochid_client_session_handler gzochid_client_session_handler;
 
-struct _gzochid_client_session
-{
-  gzochid_auth_identity *identity;
-  gzochid_client_session_handler *handler;
-  gboolean connected;
-
-  GSequence *channels;
-
-  mpz_t scm_oid;
-};
-
 typedef struct _gzochid_client_session gzochid_client_session;
 
 extern gzochid_io_serialization gzochid_client_session_serialization;
@@ -81,8 +69,41 @@ void gzochid_client_session_send_message
 (gzochid_application_context *, gzochid_client_session *, unsigned char *, 
  short);
 
+gzochid_auth_identity *gzochid_client_session_identity
+(gzochid_client_session *);
+
+/* Returns the oid of the specified session's Scheme representation. */
+
+guint64 gzochid_client_session_scm_oid (gzochid_client_session *);
+
+/* Sets the oid of the specified session's Scheme representation to the 
+   specified oid. */
+
+void gzochid_client_session_set_scm_oid (gzochid_client_session *, guint64);
+
+/* Returns the oid of the Scheme representation of the specified session's 
+   session listener. */
+
+guint64 gzochid_client_session_handler_scm_oid (gzochid_client_session *);
+
+/* Sets the oid of the Scheme representation of the specified session's session
+   listener to the specified oid. */
+
+void gzochid_client_session_set_handler_scm_oid
+(gzochid_client_session *, guint64);
+
+/* Returns the lifecycle handler of the specified channel. */
+
+gzochid_client_session_handler *gzochid_client_session_get_handler
+(gzochid_client_session *);
+
+/* Sets the lifecycle handler for the specified channel. */
+
+void gzochid_client_session_set_handler
+(gzochid_client_session *, gzochid_client_session_handler *);
+
 void gzochid_client_session_persist 
-(gzochid_application_context *, gzochid_client_session *, mpz_t);
+(gzochid_application_context *, gzochid_client_session *, guint64 *, GError **);
 void gzochid_sweep_client_sessions (gzochid_application_context *, GError **);
 
 #endif /* GZOCHID_SESSION_H */
