@@ -1,5 +1,5 @@
 /* test-dataserver-protocol.c: Test routines for test-dataserver-protocol.c.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -20,16 +20,34 @@
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/socket.h>
+#include <unistd.h>
 
 #include "data-protocol.h"
 #include "dataserver.h"
 #include "dataserver-protocol.h"
+#include "event.h"
+#include "event-meta.h"
 #include "protocol.h"
 #include "resolver.h"
 #include "socket.h"
 
+struct _GzochiMetadDataServer
+{
+  GObject parent_instance;
+};
+
+G_DEFINE_TYPE (GzochiMetadDataServer, gzochi_metad_data_server, G_TYPE_OBJECT);
+
+static void
+gzochi_metad_data_server_class_init (GzochiMetadDataServerClass *klass)
+{
+}
+
+static void
+gzochi_metad_data_server_init (GzochiMetadDataServer *self)
+{
+}
+
 static GList *activity_log = NULL;
 
 static void
@@ -41,7 +59,7 @@ clear_activity_log ()
 
 gzochid_data_reserve_oids_response *
 gzochi_metad_dataserver_reserve_oids (GzochiMetadDataServer *dataserver,
-				      guint node_id, char *app)
+				      guint node_id, const char *app)
 {
   gzochid_data_oids_block block;
   gzochid_data_reserve_oids_response *response = NULL;
@@ -56,9 +74,9 @@ gzochi_metad_dataserver_reserve_oids (GzochiMetadDataServer *dataserver,
 
 gzochid_data_response *
 gzochi_metad_dataserver_request_value (GzochiMetadDataServer *dataserver,
-				       guint node_id, char *app, char *store,
-				       GBytes *key, gboolean for_write,
-				       GError **err)
+				       guint node_id, const char *app,
+				       const char *store, GBytes *key,
+				       gboolean for_write, GError **err)
 {
   GBytes *data = g_bytes_new_static ("foo", 4);
   gzochid_data_response *response =
@@ -70,8 +88,9 @@ gzochi_metad_dataserver_request_value (GzochiMetadDataServer *dataserver,
 
 gzochid_data_response *
 gzochi_metad_dataserver_request_next_key (GzochiMetadDataServer *dataserver,
-					  guint node_id, char *app, char *store,
-					  GBytes *key, GError **err)
+					  guint node_id, const char *app,
+					  const char *store, GBytes *key,
+					  GError **err)
 {
   GBytes *data = g_bytes_new_static ("foo1", 5);
   gzochid_data_response *response = gzochid_data_response_new
@@ -83,8 +102,8 @@ gzochi_metad_dataserver_request_next_key (GzochiMetadDataServer *dataserver,
 
 void
 gzochi_metad_dataserver_release_key (GzochiMetadDataServer *dataserver,
-				     guint node_id, char *app, char *store,
-				     GBytes *key)
+				     guint node_id, const char *app,
+				     const char *store, GBytes *key)
 {
   activity_log = g_list_append
     (activity_log, g_strdup_printf
@@ -94,8 +113,9 @@ gzochi_metad_dataserver_release_key (GzochiMetadDataServer *dataserver,
 
 void
 gzochi_metad_dataserver_release_range (GzochiMetadDataServer *dataserver,
-				       guint node_id, char *app, char *store,
-				       GBytes *from, GBytes *to)
+				       guint node_id, const char *app,
+				       const char *store, GBytes *from,
+				       GBytes *to)
 {
   activity_log = g_list_append
     (activity_log, g_strdup_printf
@@ -137,57 +157,95 @@ gzochi_metad_dataserver_process_changeset (GzochiMetadDataServer *dataserver,
     }
 }
 
+struct _metaserver_wrapper_client
+{
+  gzochi_metad_dataserver_client *dataserver_client;
+};
+
+typedef struct _metaserver_wrapper_client metaserver_wrapper_client;
+
+static gboolean
+can_dispatch_wrapper (const GByteArray *bytes, gpointer data)
+{
+  metaserver_wrapper_client *wrapper_client = data;
+
+  return gzochi_metad_dataserver_client_protocol.can_dispatch
+    (bytes, wrapper_client->dataserver_client);
+}
+
+static unsigned int
+dispatch_wrapper (const GByteArray *bytes, gpointer data)
+{
+  metaserver_wrapper_client *wrapper_client = data;
+
+  return gzochi_metad_dataserver_client_protocol.dispatch
+    (bytes, wrapper_client->dataserver_client);
+}
+
+static void
+error_wrapper (gpointer data)
+{
+  metaserver_wrapper_client *wrapper_client = data;
+
+  gzochi_metad_dataserver_client_protocol.error
+    (wrapper_client->dataserver_client);
+}
+
+static void
+free_wrapper (gpointer data)
+{
+  metaserver_wrapper_client *wrapper_client = data;
+
+  gzochi_metad_dataserver_client_protocol.free
+    (wrapper_client->dataserver_client);
+  free (wrapper_client);
+}
+
+static gzochid_client_protocol wrapper_protocol =
+  { can_dispatch_wrapper, dispatch_wrapper, error_wrapper, free_wrapper };
+
 struct _dataserver_protocol_fixture
 {
   GzochidSocketServer *socket_server;
-  gzochid_client_socket *client_socket;
-  gzochid_server_socket *server_socket;
+  GzochiMetadDataServer *dataserver;
+  metaserver_wrapper_client *client;
 
-  GIOChannel *socket_channel;
+  GIOChannel *read_channel;
+  GIOChannel *write_channel;
 };
 
 typedef struct _dataserver_protocol_fixture dataserver_protocol_fixture;
 
-static gzochid_client_socket *
-server_accept_wrapper (GIOChannel *channel, const char *desc, gpointer data)
-{
-  dataserver_protocol_fixture *fixture = data;
-  gzochid_client_socket *ret = gzochi_metad_dataserver_server_protocol.accept
-    (channel, desc, NULL);
-
-  fixture->client_socket = ret;
-  return ret;
-}
-
-static gzochid_server_protocol dataserver_server_wrapper_protocol =
-  { server_accept_wrapper };
-
 static void
 dataserver_protocol_fixture_set_up (dataserver_protocol_fixture *fixture,
 				    gconstpointer user_data)
 {
-  struct sockaddr addr;
-  size_t addrlen = sizeof (struct sockaddr);
-  int socket_fd = socket (AF_INET, SOCK_STREAM, 0);
-
+  int pipefd[2] = { 0 };
+  int pipe_fd = pipe (pipefd);
+  GIOChannel *write_channel = g_io_channel_unix_new (pipefd[1]);
+  metaserver_wrapper_client *wrapper_client =
+    malloc (sizeof (metaserver_wrapper_client));
+  gzochid_client_socket *client_socket = gzochid_client_socket_new
+    (write_channel, "", wrapper_protocol, wrapper_client);
+  
   fixture->socket_server = gzochid_resolver_require
     (GZOCHID_TYPE_SOCKET_SERVER, NULL);
+  fixture->dataserver = gzochid_resolver_require
+    (GZOCHI_METAD_TYPE_DATA_SERVER, NULL);
+
+  fixture->read_channel = g_io_channel_unix_new (pipefd[0]);
+  fixture->write_channel = write_channel;
+  fixture->client = wrapper_client;
   
-  fixture->server_socket = gzochid_server_socket_new
-    ("test", dataserver_server_wrapper_protocol, fixture);
+  wrapper_client->dataserver_client = gzochi_metad_dataserver_client_new
+    (fixture->dataserver, client_socket, 1);
+
+  g_io_channel_set_flags (fixture->read_channel, G_IO_FLAG_NONBLOCK, NULL);  
+  g_io_channel_set_encoding (write_channel, NULL, NULL);
+  g_io_channel_set_flags (write_channel, G_IO_FLAG_NONBLOCK, NULL);
   
-  gzochid_server_socket_listen
-    (fixture->socket_server, fixture->server_socket, 0);
-  _gzochid_server_socket_getsockname (fixture->server_socket, &addr, &addrlen);
-  connect (socket_fd, &addr, addrlen);
-
-  g_assert (g_main_context_iteration
-	    (fixture->socket_server->main_context, TRUE));
-
-  g_assert (fixture->client_socket != NULL);
-
-  fixture->socket_channel = g_io_channel_unix_new (socket_fd);
-  g_io_channel_set_flags (fixture->socket_channel, G_IO_FLAG_NONBLOCK, NULL);
+  gzochid_client_socket_listen (fixture->socket_server, client_socket);
+  gzochid_client_socket_unref (client_socket);
 }
 
 static void
@@ -195,35 +253,22 @@ dataserver_protocol_fixture_tear_down (dataserver_protocol_fixture *fixture,
 				       gconstpointer user_data)
 {
   g_object_unref (fixture->socket_server);
-  g_io_channel_unref (fixture->socket_channel);
-
+  g_object_unref (fixture->dataserver);
+  
+  g_io_channel_unref (fixture->read_channel);
+  
   clear_activity_log ();
 }
 
-static void
-test_server_accept (dataserver_protocol_fixture *fixture,
-		    gconstpointer user_data)
-{
-  gzochid_client_protocol protocol =
-    _gzochid_client_socket_get_protocol (fixture->client_socket);
-
-  g_assert (memcmp (&protocol, &gzochi_metad_dataserver_client_protocol,
-		    sizeof (gzochid_client_protocol)) == 0);
-}
-
 static void
 test_client_can_dispatch_true (dataserver_protocol_fixture *fixture,
 			       gconstpointer user_data)
 {
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
   GByteArray *bytes = g_byte_array_new ();
 
-  g_byte_array_append (bytes, "\x00\x18\x10\x01http://localhost:8080/", 27);
+  g_byte_array_append (bytes, "\x00\x18\x10\x02http://localhost:8080/", 27);
 
-  g_assert
-    (gzochi_metad_dataserver_client_protocol.can_dispatch (bytes, client));
+  g_assert (wrapper_protocol.can_dispatch (bytes, fixture->client));
 
   g_byte_array_unref (bytes);
 }
@@ -232,55 +277,31 @@ static void
 test_client_can_dispatch_false (dataserver_protocol_fixture *fixture,
 				gconstpointer user_data)
 {
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
   GByteArray *bytes = g_byte_array_new ();
 
-  g_byte_array_append (bytes, "\x00\x18\x10\x01http", 9);
+  g_byte_array_append (bytes, "\x00\x18\x10\x02http", 9);
 
-  g_assert
-    (! gzochi_metad_dataserver_client_protocol.can_dispatch (bytes, client));
+  g_assert (! wrapper_protocol.can_dispatch (bytes, fixture->client));
 
   g_byte_array_unref (bytes);
 }
 
-static void
-test_client_dispatch_one_login (dataserver_protocol_fixture *fixture,
-				gconstpointer user_data)
-{
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
-  GByteArray *bytes = g_byte_array_new ();
-
-  g_byte_array_append (bytes, "\x00\x18\x10\x01http://localhost:8080/", 27);
-  gzochi_metad_dataserver_client_protocol.dispatch (bytes, client);
-  g_byte_array_unref (bytes);
-
-  g_assert_cmpstr
-    (gzochi_metad_dataserver_client_get_admin_server_base_url (client), ==,
-     "http://localhost:8080/");
-}
-
 static void
 test_client_dispatch_one_reserve_oids (dataserver_protocol_fixture *fixture,
 				       gconstpointer user_data)
 {
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
   char buf[18];
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append (bytes, "\x00\x05\x20test", 8);
-  gzochi_metad_dataserver_client_protocol.dispatch (bytes, client);
+  wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
   g_main_context_iteration (fixture->socket_server->main_context, FALSE);
+  g_io_channel_flush (fixture->write_channel, NULL);
   
   g_assert_cmpint
-    (g_io_channel_read_chars (fixture->socket_channel, buf, 18, NULL, NULL), ==,
+    (g_io_channel_read_chars (fixture->read_channel, buf, 18, NULL, NULL), ==,
      G_IO_STATUS_NORMAL);
 
   g_assert(memcmp (buf,
@@ -292,22 +313,20 @@ static void
 test_client_dispatch_one_request_value (dataserver_protocol_fixture *fixture,
 					gconstpointer user_data)
 {
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
   char buf[20];
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append
     (bytes, "\x00\x0f\x21test\x00oids\x00\x00\x00\x02""1", 18);
   
-  gzochi_metad_dataserver_client_protocol.dispatch (bytes, client);
+  wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
   g_main_context_iteration (fixture->socket_server->main_context, FALSE);
-  
+  g_io_channel_flush (fixture->write_channel, NULL);
+
   g_assert_cmpint
-    (g_io_channel_read_chars (fixture->socket_channel, buf, 20, NULL, NULL), ==,
+    (g_io_channel_read_chars (fixture->read_channel, buf, 20, NULL, NULL), ==,
      G_IO_STATUS_NORMAL);
   
   g_assert
@@ -319,20 +338,18 @@ static void
 test_client_dispatch_one_request_next_key
 (dataserver_protocol_fixture *fixture, gconstpointer user_data)
 {
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
   char buf[22];
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append (bytes, "\x00\x11\x22test\x00names\x00\x00\x04""foo", 20);
-  gzochi_metad_dataserver_client_protocol.dispatch (bytes, client);
+  wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
   g_main_context_iteration (fixture->socket_server->main_context, FALSE);
-  
+  g_io_channel_flush (fixture->write_channel, NULL);
+
   g_assert_cmpint
-    (g_io_channel_read_chars (fixture->socket_channel, buf, 22, NULL, NULL), ==,
+    (g_io_channel_read_chars (fixture->read_channel, buf, 22, NULL, NULL), ==,
      G_IO_STATUS_NORMAL);
   
   g_assert
@@ -344,13 +361,10 @@ static void
 test_client_dispatch_one_release_key (dataserver_protocol_fixture *fixture,
 				      gconstpointer user_data)
 {
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append (bytes, "\x00\x0e\x40test\x00oids\x00\x00\x02""1", 17);
-  gzochi_metad_dataserver_client_protocol.dispatch (bytes, client);
+  wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
   g_assert_cmpint (g_list_length (activity_log), ==, 1);
@@ -361,15 +375,12 @@ static void
 test_client_dispatch_one_release_range
 (dataserver_protocol_fixture *fixture, gconstpointer user_data)
 {
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append
     (bytes, "\x00\x19\x42test\x00names\x00\x00\x05""foo1\x00\x00\x05""foo2\x00",
      28);
-  gzochi_metad_dataserver_client_protocol.dispatch (bytes, client);
+  wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
   g_assert_cmpint (g_list_length (activity_log), ==, 1);
@@ -381,9 +392,6 @@ static void
 test_client_dispatch_one_process_changeset
 (dataserver_protocol_fixture *fixture, gconstpointer user_data)
 {
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append (bytes, "\x00\x3f\x30test", 8);
@@ -393,7 +401,7 @@ test_client_dispatch_one_process_changeset
   g_byte_array_append (bytes, "names\x00\x00\x04""foo\x00\x00\x00", 14);
   g_byte_array_append (bytes, "names\x00\x00\x04""bar\x00\x00\x02""3", 16);
 
-  gzochi_metad_dataserver_client_protocol.dispatch (bytes, client);
+  wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
   g_assert_cmpint (g_list_length (activity_log), ==, 4);
@@ -411,16 +419,13 @@ static void
 test_client_dispatch_multiple (dataserver_protocol_fixture *fixture,
 			       gconstpointer user_data)
 {
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append
     (bytes, "\x00\x11\x40test\x00names\x00""\x00\x04""foo", 20);
   g_byte_array_append
     (bytes, "\x00\x11\x40test\x00names\x00""\x00\x04""bar", 20);
-  gzochi_metad_dataserver_client_protocol.dispatch (bytes, client);
+  wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
   g_assert_cmpint (g_list_length (activity_log), ==, 2);
@@ -434,17 +439,10 @@ static void
 test_client_error (dataserver_protocol_fixture *fixture,
 		   gconstpointer user_data)
 {
-  GSource *source = g_main_context_find_source_by_user_data
-    (fixture->socket_server->main_context, fixture->client_socket);
-  gzochi_metad_dataserver_client *client =
-    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
-
-  gzochi_metad_dataserver_client_protocol.error (client);
+  wrapper_protocol.error (fixture->client);
 
   g_assert_cmpint (g_list_length (activity_log), ==, 1);
   g_assert_cmpstr ((char *) activity_log->data, ==, "RELEASE ALL");
-
-  g_source_unref (source);
 }
 
 int
@@ -458,9 +456,6 @@ main (int argc, char *argv[])
 
   g_test_init (&argc, &argv, NULL);
 
-  g_test_add ("/server/accept", dataserver_protocol_fixture, NULL,
-	      dataserver_protocol_fixture_set_up, test_server_accept,
-	      dataserver_protocol_fixture_tear_down);
   g_test_add ("/client/can_dispatch/true", dataserver_protocol_fixture, NULL,
 	      dataserver_protocol_fixture_set_up, test_client_can_dispatch_true,
 	      dataserver_protocol_fixture_tear_down);
@@ -470,11 +465,6 @@ main (int argc, char *argv[])
      dataserver_protocol_fixture_set_up, test_client_can_dispatch_false,
      dataserver_protocol_fixture_tear_down);
 
-  g_test_add
-    ("/client/dispatch/one/login", dataserver_protocol_fixture, NULL,
-     dataserver_protocol_fixture_set_up, test_client_dispatch_one_login,
-     dataserver_protocol_fixture_tear_down);
-
   g_test_add
     ("/client/dispatch/one/reserve-oids", dataserver_protocol_fixture, NULL,
      dataserver_protocol_fixture_set_up, test_client_dispatch_one_reserve_oids,
@@ -499,7 +489,7 @@ main (int argc, char *argv[])
      dataserver_protocol_fixture_tear_down);
 
   g_test_add
-    ("/client/dispatch/release-range", dataserver_protocol_fixture, NULL,
+    ("/client/dispatch/one/release-range", dataserver_protocol_fixture, NULL,
      dataserver_protocol_fixture_set_up, test_client_dispatch_one_release_range,
      dataserver_protocol_fixture_tear_down);
 
