@@ -1,5 +1,5 @@
 /* test-game-protocol.c: Test routines for game-protocol.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,13 +22,17 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 
+#include "config.h"
 #include "game.h"
 #include "game-protocol.h"
+#include "resolver.h"
 #include "socket.h"
 
 struct _game_protocol_fixture
 {
-  gzochid_game_context *game_context;
+  GzochidResolutionContext *resolution_context;
+  GzochidGameServer *game_server;
+  GzochidSocketServer *socket_server;
 
   gzochid_client_socket *client_socket;
   gzochid_server_socket *server_socket;
@@ -51,7 +55,7 @@ server_accept_wrapper (GIOChannel *channel, const char *desc, gpointer data)
 {
   game_protocol_fixture *fixture = data;
   gzochid_client_socket *ret = gzochid_game_server_protocol.accept
-    (channel, desc, fixture->game_context);
+    (channel, desc, fixture->game_server);
 
   fixture->client_socket = ret;
   return ret;
@@ -67,46 +71,53 @@ game_protocol_fixture_set_up (game_protocol_fixture *fixture,
   struct sockaddr addr;
   size_t addrlen = sizeof (struct sockaddr);
   int socket_fd = socket (AF_INET, SOCK_STREAM, 0);
-
+  GKeyFile *key_file = g_key_file_new ();
+  GzochidConfiguration *configuration = g_object_new
+    (GZOCHID_TYPE_CONFIGURATION, "key_file", key_file, NULL);
+  
   g_test_log_set_fatal_handler (ignore_warnings, NULL);
   
-  fixture->game_context = gzochid_game_context_new ();
-  g_mutex_init (&((gzochid_context *) fixture->game_context)->mutex);
+  fixture->resolution_context = g_object_new
+    (GZOCHID_TYPE_RESOLUTION_CONTEXT, NULL);
+
+  gzochid_resolver_provide
+    (fixture->resolution_context, G_OBJECT (configuration), NULL);
+  
+  fixture->game_server = gzochid_resolver_require_full
+    (fixture->resolution_context, GZOCHID_TYPE_GAME_SERVER, NULL);
+  fixture->socket_server = gzochid_resolver_require_full
+    (fixture->resolution_context, GZOCHID_TYPE_SOCKET_SERVER, NULL);
   
-  fixture->game_context->socket_server = g_object_new
-    (GZOCHID_TYPE_SOCKET_SERVER, NULL);
   fixture->server_socket = gzochid_server_socket_new
     ("test", game_server_wrapper_protocol, fixture);
   
   gzochid_server_socket_listen
-    (fixture->game_context->socket_server, fixture->server_socket, 0);
+    (fixture->socket_server, fixture->server_socket, 0);
   _gzochid_server_socket_getsockname (fixture->server_socket, &addr, &addrlen);
   connect (socket_fd, &addr, addrlen);
 
-  g_assert
-    (g_main_context_iteration
-     (fixture->game_context->socket_server->main_context, FALSE));
+  g_assert (g_main_context_iteration
+	    (fixture->socket_server->main_context, FALSE));
 
   g_assert (fixture->client_socket != NULL);
+
+  g_key_file_unref (key_file);
+  g_object_unref (configuration);
 }
 
 static void
 game_protocol_fixture_tear_down (game_protocol_fixture *fixture,
 				 gconstpointer user_data)
 {
-  GSource *server_source = g_main_context_find_source_by_user_data
-    (fixture->game_context->socket_server->main_context,
-     fixture->server_socket);
   GSource *client_source = g_main_context_find_source_by_user_data
-    (fixture->game_context->socket_server->main_context,
-     fixture->client_socket);
+    (fixture->socket_server->main_context, fixture->client_socket);
 
-  g_source_unref (server_source);
-  
   if (client_source != NULL)
     g_source_unref (client_source);
-  
-  gzochid_game_context_free (fixture->game_context);
+
+  g_object_unref (fixture->game_server);
+  g_object_unref (fixture->resolution_context);
+  g_object_unref (fixture->socket_server);
 }
 
 static void
@@ -201,8 +212,7 @@ static void
 test_client_error (game_protocol_fixture *fixture, gconstpointer user_data)
 {
   GSource *source = g_main_context_find_source_by_user_data
-    (fixture->game_context->socket_server->main_context,
-     fixture->client_socket);
+    (fixture->socket_server->main_context, fixture->client_socket);
   gzochid_game_client *client = _gzochid_client_socket_get_protocol_data
     (fixture->client_socket);
 
