@@ -1,5 +1,5 @@
 /* test-scheme-task.c: Test routines for scheme-task.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,18 +16,21 @@
  */
 
 #include <glib.h>
+#include <glib-object.h>
 #include <libguile.h>
 #include <stddef.h>
 #include <stdlib.h>
 
 #include "app.h"
 #include "app-task.h"
-#include "context.h"
+#include "config.h"
+#include "descriptor.h"
 #include "game.h"
 #include "guile.h"
 #include "gzochid-auth.h"
 #include "oids.h"
 #include "oids-storage.h"
+#include "resolver.h"
 #include "scheme.h"
 #include "scheme-task.h"
 #include "storage-mem.h"
@@ -37,7 +40,10 @@
 
 struct test_scheme_task_fixture
 {
-  gzochid_application_context *context;
+  GzochidResolutionContext *resolution_context;
+  GzochidGameServer *game_server;
+  
+  gzochid_application_context *app_context;
   gzochid_auth_identity *identity;
   gzochid_storage_engine_interface *storage_interface;
 };
@@ -56,64 +62,69 @@ static void
 test_scheme_task_fixture_setup (struct test_scheme_task_fixture *fixture,
 				gconstpointer user_data)
 {
-  gzochid_context *base_context = NULL;
-  gzochid_game_context *game_context = gzochid_game_context_new (NULL);
-  gzochid_storage_engine *storage_engine = 
-    malloc (sizeof (gzochid_storage_engine));
+  GKeyFile *key_file = g_key_file_new ();
+  GzochidConfiguration *configuration = g_object_new
+    (GZOCHID_TYPE_CONFIGURATION, "key_file", key_file, NULL);
 
-  fixture->context = gzochid_application_context_new ();
-  fixture->context->deployment_root = "/tmp";
-  fixture->context->descriptor = 
-    calloc (1, sizeof (gzochid_application_descriptor));
-  fixture->context->identity_cache = gzochid_auth_identity_cache_new ();
+  fixture->app_context = gzochid_application_context_new ();
+  fixture->app_context->deployment_root = "/tmp";
+  fixture->app_context->descriptor = g_object_new
+    (GZOCHID_TYPE_APPLICATION_DESCRIPTOR, NULL);
+  fixture->app_context->identity_cache = gzochid_auth_identity_cache_new ();
 
   fixture->identity = gzochid_auth_identity_new ("[TEST]");
-  
-  base_context = (gzochid_context *) fixture->context;
-  base_context->parent = (gzochid_context *) game_context;
 
-  g_mutex_init (&base_context->mutex);
-  g_mutex_init (&base_context->parent->mutex);
+  fixture->resolution_context = g_object_new
+    (GZOCHID_TYPE_RESOLUTION_CONTEXT, NULL);
+
+  gzochid_resolver_provide
+    (fixture->resolution_context, G_OBJECT (configuration), NULL);
+
+  fixture->game_server = gzochid_resolver_require_full
+    (fixture->resolution_context, GZOCHID_TYPE_GAME_SERVER, NULL);
   
-  game_context->storage_engine = storage_engine;
-  storage_engine->interface = &gzochid_storage_engine_interface_mem;
+  fixture->storage_interface = &gzochid_storage_engine_interface_mem;
   
-  fixture->storage_interface = storage_engine->interface;
-  
-  fixture->context->storage_context =
+  fixture->app_context->storage_engine_interface = fixture->storage_interface;
+  fixture->app_context->storage_context =
     fixture->storage_interface->initialize ("/tmp");
-  fixture->context->meta = fixture->storage_interface->open
-    (fixture->context->storage_context, "/tmp/meta", GZOCHID_STORAGE_CREATE);
-  fixture->context->oids = fixture->storage_interface->open
-    (fixture->context->storage_context, "/tmp/oids", GZOCHID_STORAGE_CREATE);
-  fixture->context->names = fixture->storage_interface->open
-    (fixture->context->storage_context, "/tmp/names", GZOCHID_STORAGE_CREATE);
+  fixture->app_context->meta = fixture->storage_interface->open
+    (fixture->app_context->storage_context, "/tmp/meta",
+     GZOCHID_STORAGE_CREATE);
+  fixture->app_context->oids = fixture->storage_interface->open
+    (fixture->app_context->storage_context, "/tmp/oids",
+     GZOCHID_STORAGE_CREATE);
+  fixture->app_context->names = fixture->storage_interface->open
+    (fixture->app_context->storage_context, "/tmp/names",
+     GZOCHID_STORAGE_CREATE);
 
-  fixture->context->oid_strategy = gzochid_storage_oid_strategy_new
-    (fixture->storage_interface, fixture->context->storage_context,
-     fixture->context->meta);
+  fixture->app_context->oid_strategy = gzochid_storage_oid_strategy_new
+    (fixture->storage_interface, fixture->app_context->storage_context,
+     fixture->app_context->meta);
+
+  g_key_file_unref (key_file);
+  g_object_unref (configuration);
 }
 
 static void
 test_scheme_task_fixture_teardown (struct test_scheme_task_fixture *fixture,
 				   gconstpointer user_data)
 {
-  gzochid_game_context *game_context = (gzochid_game_context *)
-    ((gzochid_context *) fixture->context)->parent;
-    
-  free (fixture->context->descriptor);
-  gzochid_auth_identity_cache_destroy (fixture->context->identity_cache);
+  g_object_unref (fixture->app_context->descriptor);
+  gzochid_auth_identity_cache_destroy (fixture->app_context->identity_cache);
 
-  fixture->storage_interface->close_store (fixture->context->meta);
-  fixture->storage_interface->close_store (fixture->context->oids);
-  fixture->storage_interface->close_store (fixture->context->names);
-  fixture->storage_interface->close_context (fixture->context->storage_context);
+  fixture->storage_interface->close_store (fixture->app_context->meta);
+  fixture->storage_interface->close_store (fixture->app_context->oids);
+  fixture->storage_interface->close_store (fixture->app_context->names);
+  fixture->storage_interface->close_context
+    (fixture->app_context->storage_context);
 
-  gzochid_oid_allocation_strategy_free (fixture->context->oid_strategy);
-  
-  free (game_context->storage_engine);
-  gzochid_game_context_free (game_context);
-  gzochid_application_context_free (fixture->context);
+  gzochid_oid_allocation_strategy_free (fixture->app_context->oid_strategy);
+
+  g_object_unref (fixture->game_server);
+  g_object_unref (fixture->resolution_context);
+
+  gzochid_application_context_free (fixture->app_context);
   gzochid_auth_identity_unref (fixture->identity);
 }
 
@@ -124,15 +135,15 @@ persist_client_session (struct test_scheme_task_fixture *fixture,
   GByteArray *out = g_byte_array_new ();
 
   gzochid_storage_transaction *tx = fixture->storage_interface
-    ->transaction_begin (fixture->context->storage_context);
+    ->transaction_begin (fixture->app_context->storage_context);
   
   gzochid_client_session_serialization.serializer 
-    (fixture->context, session, out, NULL);
+    (fixture->app_context, session, out, NULL);
   
   fixture->storage_interface->transaction_put
-    (tx, fixture->context->oids, "1", 2, out->data, out->len);
+    (tx, fixture->app_context->oids, "1", 2, out->data, out->len);
   fixture->storage_interface->transaction_put
-    (tx, fixture->context->names, "s.session.1", 12, "1", 2);
+    (tx, fixture->app_context->names, "s.session.1", 12, "1", 2);
 
   fixture->storage_interface->transaction_prepare (tx);
   fixture->storage_interface->transaction_commit (tx);
@@ -148,7 +159,7 @@ test_disconnected_worker_no_handler_inner (gpointer data)
   g_test_log_set_fatal_handler (ignore_warnings, NULL);
   
   gzochid_scheme_application_disconnected_worker
-    (fixture->context, fixture->identity, "1");
+    (fixture->app_context, fixture->identity, "1");
 }
 
 static void 
@@ -166,11 +177,11 @@ test_disconnected_worker_no_handler (struct test_scheme_task_fixture *fixture,
     (test_disconnected_worker_no_handler_inner, fixture);
 
   tx = fixture->storage_interface->transaction_begin
-    (fixture->context->storage_context);
+    (fixture->app_context->storage_context);
 
   g_assert
     (fixture->storage_interface->transaction_get
-     (tx, fixture->context->oids, (char *) &one, sizeof (guint64), NULL) ==
+     (tx, fixture->app_context->oids, (char *) &one, sizeof (guint64), NULL) ==
      NULL);
 
   fixture->storage_interface->transaction_rollback (tx);
@@ -209,14 +220,13 @@ test_logged_in_worker_throws_exception_inner (gpointer data)
 
   scm_c_module_define (module, "logged-in-exception", logged_in);
 
-  fixture->context->descriptor->logged_in = make_callback 
+  fixture->app_context->descriptor->logged_in = make_callback 
     ("logged-in-exception", test_module);
   
   gzochid_scheme_application_logged_in_worker
-    (fixture->context, fixture->identity, "1");
+    (fixture->app_context, fixture->identity, "1");
 
   g_list_free (test_module);
-  gzochid_application_callback_free (fixture->context->descriptor->logged_in);
 }
 
 static void 
@@ -246,14 +256,13 @@ test_logged_in_worker_returns_unspecified_inner (gpointer data)
 
   scm_c_module_define (module, "logged-in-unspecified", logged_in);
 
-  fixture->context->descriptor->logged_in = make_callback 
+  fixture->app_context->descriptor->logged_in = make_callback 
     ("logged-in-unspecified", test_module);
 
   gzochid_scheme_application_logged_in_worker
-    (fixture->context, fixture->identity, "1");
+    (fixture->app_context, fixture->identity, "1");
 
   g_list_free (test_module);
-  gzochid_application_callback_free (fixture->context->descriptor->logged_in);
 }
 
 static void 
@@ -274,8 +283,8 @@ static void
 test_ready_throws_exception ()
 {
   gzochid_application_context *context = gzochid_application_context_new ();
-  gzochid_application_descriptor *descriptor = 
-    calloc (1, sizeof (gzochid_application_descriptor));
+  GzochidApplicationDescriptor *descriptor = g_object_new
+    (GZOCHID_TYPE_APPLICATION_DESCRIPTOR, NULL);
   gzochid_auth_identity *identity = gzochid_auth_identity_new ("[TEST]");
   
   SCM module = scm_c_resolve_module ("test");
@@ -284,11 +293,8 @@ test_ready_throws_exception ()
 
   GError *tmp_err = NULL;
 
-  g_mutex_init (&((gzochid_context *) context)->mutex);
-  
   scm_c_module_define (module, "ready", ready);
 
-  descriptor->properties = g_hash_table_new (g_str_hash, g_str_equal);
   descriptor->ready = make_callback ("ready", test_module);
 
   context->deployment_root = "/";
@@ -300,10 +306,8 @@ test_ready_throws_exception ()
 
   g_error_free (tmp_err);
   g_list_free (test_module);
-  gzochid_application_callback_free (descriptor->ready);
   gzochid_auth_identity_unref (identity);
-  g_hash_table_destroy (descriptor->properties);
-  free (descriptor);
+  g_object_unref (descriptor);
   gzochid_application_context_free (context);
 }
 
