@@ -1,5 +1,5 @@
 /* test-storage-dataclient.c: Tests for storage-dataclient.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -15,11 +15,13 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <assert.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <sys/time.h>
 
 #include "dataclient.h"
@@ -48,51 +50,125 @@ struct _dataclient_storage_response_closure
 
   gzochid_dataclient_failure_callback failure_callback;
   gpointer failure_data;
+
+  gchar *qualified_key;
+  gzochid_dataclient_release_callback release_callback;
+  gpointer release_data;
 };
 
 typedef struct _dataclient_storage_response_closure
 dataclient_storage_response_closure;
 
+struct _dataclient_release_closure
+{
+  gchar *qualified_key;
+  gzochid_dataclient_release_callback release_callback;
+  gpointer release_data;
+};
+
+typedef struct _dataclient_release_closure dataclient_release_closure;
+
 struct _GzochidDataClient
 {
   GObject parent_instance;
 
+  GMutex process_response_mutex;
+  GCond process_response_cond;
+  
   GList *responses;
   GList *requested_keys;
-  GList *requested_values;
+  GList *released_keys;
   GList *changesets;
   GList *deferred_response_closures;
+  GList *release_closures;
+  GThread *processing_thread;
 };
 
 G_DEFINE_TYPE (GzochidDataClient, gzochid_data_client, G_TYPE_OBJECT);
 
+static void
+gzochid_data_client_finalize (GObject *object)
+{
+  GzochidDataClient *self = GZOCHID_DATA_CLIENT (object);
+
+  g_mutex_clear (&self->process_response_mutex);
+  g_cond_clear (&self->process_response_cond);
+}
+
 static void
 gzochid_data_client_class_init (GzochidDataClientClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->finalize = gzochid_data_client_finalize;
+}
+
+static void
+execute_release_closure (gpointer data)
+{
+  dataclient_release_closure *closure = data;
+  closure->release_callback (closure->release_data);
+
+  g_free (closure->qualified_key);
+  free (closure);
 }
 
 static void
 gzochid_data_client_init (GzochidDataClient *self)
 {
+  g_mutex_init (&self->process_response_mutex);
+  g_cond_init (&self->process_response_cond);
+  
   self->responses = NULL;
   self->requested_keys = NULL;
-  self->requested_values = NULL;
+  self->released_keys = NULL;
   self->changesets = NULL;
   self->deferred_response_closures = NULL;
+  self->release_closures = NULL;
+}
+
+static dataclient_release_closure *
+create_release_closure (gchar *qualified_key,
+			gzochid_dataclient_release_callback release_callback,
+			gpointer release_data)
+{
+  dataclient_release_closure *closure =
+    malloc (sizeof (dataclient_release_closure));
+
+  closure->qualified_key = g_strdup (qualified_key);
+  closure->release_callback = release_callback;
+  closure->release_data = release_data;
+  
+  return closure;
 }
 
 static gpointer
 process_response_async (gpointer data)
 {
   dataclient_storage_response_closure *closure = data;
-
+  GzochidDataClient *client = closure->client;
+  
   if (closure->response->success)
-    closure->success_callback (closure->response->data, closure->success_data);
+    {
+      closure->success_callback
+	(closure->response->data, closure->success_data);
+
+      client->release_closures =
+	g_list_append (client->release_closures, 
+		       create_release_closure (closure->qualified_key,
+					       closure->release_callback,
+					       closure->release_data));      
+    }
   else closure->failure_callback
 	 (closure->response->timeout, closure->failure_data);
-
+      
+  g_free (closure->qualified_key);
   free (closure);
+
+  g_mutex_lock (&client->process_response_mutex);  
+  g_cond_signal (&client->process_response_cond);
+  g_mutex_unlock (&client->process_response_mutex);
+  
   return NULL;
 }
 
@@ -107,10 +183,12 @@ execute_deferred_closure (gpointer data)
 }
 
 static dataclient_storage_response_closure *
-create_closure
-(GzochidDataClient *client, dataclient_storage_response *response,
+create_response_closure
+(GzochidDataClient *client, gchar *qualified_key,
+ dataclient_storage_response *response,
  gzochid_dataclient_success_callback success_callback, gpointer success_data,
- gzochid_dataclient_failure_callback failure_callback, gpointer failure_data)
+ gzochid_dataclient_failure_callback failure_callback, gpointer failure_data,
+ gzochid_dataclient_release_callback release_callback, gpointer release_data)
 {
   dataclient_storage_response_closure *closure =
     malloc (sizeof (dataclient_storage_response_closure));
@@ -124,6 +202,10 @@ create_closure
   closure->failure_callback = failure_callback;
   closure->failure_data = failure_data;
 
+  closure->qualified_key = g_strdup (qualified_key);
+  closure->release_callback = release_callback;
+  closure->release_data = release_data;
+  
   return closure;
 }
 
@@ -162,9 +244,10 @@ free_response (dataclient_storage_response *response)
 
 static void
 process_response
-(GzochidDataClient *client,
+(GzochidDataClient *client, gchar *qualified_key,
  gzochid_dataclient_success_callback success_callback, gpointer success_data,
- gzochid_dataclient_failure_callback failure_callback, gpointer failure_data)
+ gzochid_dataclient_failure_callback failure_callback, gpointer failure_data,
+ gzochid_dataclient_release_callback release_callback, gpointer release_data)
 {
   dataclient_storage_response_closure *closure = NULL;
   dataclient_storage_response *response = NULL;
@@ -175,17 +258,97 @@ process_response
       client->responses = g_list_delete_link
 	(client->responses, client->responses);
       
-      closure = create_closure
-	(client, response, success_callback, success_data, failure_callback,
-	 failure_data);
+      closure = create_response_closure
+	(client, qualified_key, response, success_callback, success_data,
+	 failure_callback, failure_data, release_callback, release_data);
 
-      g_thread_new ("test-response", process_response_async, closure);
+      client->processing_thread =
+	g_thread_new ("test-response", process_response_async, closure);
     }
   else client->deferred_response_closures = g_list_append
 	 (client->deferred_response_closures,
-	  create_closure
-	  (client, create_failure_response ((struct timeval) { 0, 0 }),
-	   success_callback, success_data, failure_callback, failure_data));
+	  create_response_closure
+	  (client, qualified_key,
+	   create_failure_response ((struct timeval) { 0, 0 }),
+	   success_callback, success_data, failure_callback, failure_data,
+	   release_callback, release_data));
+}
+
+static const gchar *
+key_text (GBytes *key)
+{
+  size_t size = 0;
+  const char *ret = g_bytes_get_data (key, &size);
+  assert (index (ret, '/') == NULL);
+  return ret;
+}
+
+static gchar *
+create_qualified_key (char *app, char *store, GBytes *key)
+{
+  if (key == NULL)
+    return g_strdup_printf ("/%s/%s/", app, store);
+  else return g_strdup_printf ("/%s/%s/%s", app, store, key_text (key));
+}
+
+static gchar *
+create_qualified_key_range (char *app, char *store, GBytes *from, GBytes *to)
+{
+  if (from == NULL)
+    {
+      if (to == NULL)
+	return g_strdup_printf ("/%s/%s//", app, store);
+      else return g_strdup_printf ("/%s/%s//%s", app, store, key_text (to));
+    }
+  else if (to == NULL)
+    return g_strdup_printf ("/%s/%s/%s//", app, store, key_text (from));
+  else return g_strdup_printf
+	 ("/%s/%s/%s/%s", app, store, key_text (from), key_text (to));
+}
+
+static gint
+find_release_closure (gconstpointer a, gconstpointer b)
+{
+  const dataclient_release_closure *closure = a;
+  const char *qualified_key = b;
+
+  return strcmp (closure->qualified_key, qualified_key);
+}
+
+static void
+release_key (GzochidDataClient *client, char *app, char *store, GBytes *key)
+{
+  gchar *release_closure_key = create_qualified_key (app, store, key);
+  GList *release_closure_ptr = g_list_find_custom
+    (client->release_closures, release_closure_key, find_release_closure);
+
+  if (release_closure_ptr != NULL)
+    {
+      execute_release_closure (release_closure_ptr->data);
+      client->release_closures = g_list_delete_link
+	(client->release_closures, release_closure_ptr);
+    }
+  
+  g_free (release_closure_key);
+}
+
+static void
+release_key_range (GzochidDataClient *client, char *app, char *store,
+		   GBytes *from, GBytes *to)
+{
+  gchar *release_closure_key =
+    create_qualified_key_range (app, store, from, to);
+  GList *release_closure_ptr = g_list_find_custom
+    (client->release_closures, release_closure_key, find_release_closure);
+
+  if (release_closure_ptr != NULL)
+    {
+      execute_release_closure (release_closure_ptr->data);
+      client->release_closures = g_list_delete_link
+	(client->release_closures, release_closure_ptr);
+    }
+
+  g_free (release_closure_key);
 }
 
 void
@@ -193,26 +356,35 @@ gzochid_dataclient_request_value
 (GzochidDataClient *client, char *app, char *store, GBytes *key,
  gboolean for_write, gzochid_dataclient_success_callback success_callback,
  gpointer success_data, gzochid_dataclient_failure_callback failure_callback,
- gpointer failure_data)
+ gpointer failure_data, gzochid_dataclient_release_callback release_callback,
+ gpointer release_data)
 {
-  client->requested_values =
-    g_list_append (client->requested_values, g_bytes_ref (key));
-  
+  gchar *qualified_key = create_qualified_key (app, store, key);
+
+  client->requested_keys =
+    g_list_append (client->requested_keys, qualified_key);
+
   process_response
-    (client, success_callback, success_data, failure_callback, failure_data);
+    (client, qualified_key, success_callback, success_data, failure_callback,
+     failure_data, release_callback, release_data);
 }
 
 void
 gzochid_dataclient_request_next_key
 (GzochidDataClient *client, char *app, char *store, GBytes *key,
  gzochid_dataclient_success_callback success_callback, gpointer success_data,
- gzochid_dataclient_failure_callback failure_callback, gpointer failure_data)
+ gzochid_dataclient_failure_callback failure_callback, gpointer failure_data,
+ gzochid_dataclient_release_callback release_callback, gpointer release_data)
 {
+  gchar *qualified_key_range =
+    create_qualified_key_range (app, store, key, NULL);
+  
   client->requested_keys = g_list_append
-    (client->requested_keys, key == NULL ? NULL : g_bytes_ref (key));
+    (client->requested_keys, qualified_key_range);
   
   process_response
-      (client, success_callback, success_data, failure_callback, failure_data);
+    (client, qualified_key_range, success_callback, success_data,
+     failure_callback, failure_data, release_callback, release_data);
 }
 
 void
@@ -223,6 +395,22 @@ gzochid_dataclient_submit_changeset (GzochidDataClient *client, char *app,
     (client->changesets, g_array_ref (changes));
 }
 
+void
+gzochid_dataclient_release_key (GzochidDataClient *client, char *app,
+				char *store, GBytes *key)
+{
+  client->released_keys = g_list_append
+    (client->released_keys, create_qualified_key (app, store, key));
+}
+
+void
+gzochid_dataclient_release_key_range (GzochidDataClient *client, char *app,
+				      char *store, GBytes *from, GBytes *to)
+{
+  client->released_keys = g_list_append
+    (client->released_keys, create_qualified_key_range (app, store, from, to));
+}
+
 struct _dataclient_storage_fixture
 {
   GzochidDataClient *dataclient;
@@ -274,7 +462,6 @@ clear_changeset (gpointer data)
 	g_bytes_unref (change->data);
     }
   
-  
   g_array_unref (changes);
 }
 
@@ -282,18 +469,21 @@ static void
 dataclient_storage_fixture_teardown (dataclient_storage_fixture *fixture,
 				     gconstpointer user_data)
 {
+  if (fixture->dataclient->processing_thread != NULL)
+    g_thread_join (fixture->dataclient->processing_thread);
+
   g_list_free_full
     (fixture->dataclient->deferred_response_closures, execute_deferred_closure);
-
-  g_list_free_full (fixture->dataclient->requested_keys, maybe_bytes_unref);
-  g_list_free_full
-    (fixture->dataclient->requested_values, (GDestroyNotify) g_bytes_unref);
+  g_list_free_full (fixture->dataclient->requested_keys, g_free);
   g_list_free_full (fixture->dataclient->changesets, clear_changeset);
-  
-  g_object_unref (fixture->dataclient);
+  g_list_free_full
+    (fixture->dataclient->release_closures, execute_release_closure);
+  g_list_free_full (fixture->dataclient->released_keys, g_free);
 
   fixture->iface->close_store (fixture->store);
   fixture->iface->close_context (fixture->storage_context);
+
+  g_object_unref (fixture->dataclient);
 }
 
 static void
@@ -328,8 +518,7 @@ test_get_cached (dataclient_storage_fixture *fixture, gconstpointer user_data)
   
   fixture->iface->transaction_rollback (tx);
 
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 1);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
 static void
@@ -396,8 +585,7 @@ test_get_uncached_failure_success (dataclient_storage_fixture *fixture,
   
   fixture->iface->transaction_rollback (tx);
 
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 2);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 2);
 }
 
 static void
@@ -416,8 +604,7 @@ test_get_uncached_timeout (dataclient_storage_fixture *fixture,
   
   fixture->iface->transaction_rollback (tx);
 
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 1);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
 static void
@@ -455,8 +642,7 @@ test_get_for_update_cached (dataclient_storage_fixture *fixture,
   
   fixture->iface->transaction_rollback (tx);
 
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 1);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
 static void
@@ -497,8 +683,7 @@ test_get_for_update_cached_upgrade (dataclient_storage_fixture *fixture,
   
   fixture->iface->transaction_rollback (tx);
 
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 2);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 2);
 }
 
 static void
@@ -565,8 +750,7 @@ test_get_for_update_uncached_failure_success
   
   fixture->iface->transaction_rollback (tx);
 
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 2);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 2);
 }
 
 static void
@@ -586,9 +770,7 @@ test_get_for_update_uncached_timeout (dataclient_storage_fixture *fixture,
   
   fixture->iface->transaction_rollback (tx);
 
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 1);
-
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
 static void
@@ -620,8 +802,7 @@ test_put_cached (dataclient_storage_fixture *fixture, gconstpointer user_data)
   free_response (response);
 
   g_assert_cmpint (g_list_length (fixture->dataclient->changesets), ==, 1);
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 1);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
 static void
@@ -658,8 +839,7 @@ test_put_cached_upgrade (dataclient_storage_fixture *fixture,
   free_response (response2);
 
   g_assert_cmpint (g_list_length (fixture->dataclient->changesets), ==, 1);
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 2);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 2);
 }
 
 static void
@@ -688,8 +868,7 @@ test_put_uncached_success (dataclient_storage_fixture *fixture,
   free_response (response);
   
   g_assert_cmpint (g_list_length (fixture->dataclient->changesets), ==, 1);
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 1);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
 static void
@@ -724,8 +903,7 @@ test_put_uncached_failure_success (dataclient_storage_fixture *fixture,
   free_response (response2);
   
   g_assert_cmpint (g_list_length (fixture->dataclient->changesets), ==, 1);
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 2);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 2);
 }
 
 static void
@@ -742,8 +920,7 @@ test_put_uncached_timeout (dataclient_storage_fixture *fixture,
   
   fixture->iface->transaction_rollback (tx);
 
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 1);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
 static void
@@ -779,8 +956,7 @@ test_delete_cached (dataclient_storage_fixture *fixture,
   free_response (response);
 
   g_assert_cmpint (g_list_length (fixture->dataclient->changesets), ==, 1);
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 1);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
 static void
@@ -820,8 +996,7 @@ test_delete_cached_upgrade (dataclient_storage_fixture *fixture,
   free_response (response2);
 
   g_assert_cmpint (g_list_length (fixture->dataclient->changesets), ==, 1);
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 2);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 2);
 }
 
 static void
@@ -851,8 +1026,7 @@ test_delete_uncached_success (dataclient_storage_fixture *fixture,
   free_response (response);
   
   g_assert_cmpint (g_list_length (fixture->dataclient->changesets), ==, 1);
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 1);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
 static void
@@ -888,8 +1062,7 @@ test_delete_uncached_failure_success (dataclient_storage_fixture *fixture,
   free_response (response2);
   
   g_assert_cmpint (g_list_length (fixture->dataclient->changesets), ==, 1);
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 2);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 2);
 }
 
 static void
@@ -907,8 +1080,7 @@ test_delete_uncached_timeout (dataclient_storage_fixture *fixture,
   
   fixture->iface->transaction_rollback (tx);
 
-  g_assert_cmpint
-    (g_list_length (fixture->dataclient->requested_values), ==, 1);
+  g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
 static void
@@ -1152,6 +1324,157 @@ test_next_key_uncached_timeout (dataclient_storage_fixture *fixture,
   g_assert_cmpint (g_list_length (fixture->dataclient->requested_keys), ==, 1);
 }
 
+static void
+test_lock_release_eviction (dataclient_storage_fixture *fixture,
+			    gconstpointer user_data)
+{
+  GBytes *key_bytes = g_bytes_new_static ("foo", 4);
+  GBytes *value_bytes = g_bytes_new_static ("bar", 4);
+  dataclient_storage_response *response = create_success_response (value_bytes);
+  gzochid_storage_transaction *tx = fixture->iface->transaction_begin
+    (fixture->storage_context);
+  char *val = NULL;
+  
+  fixture->dataclient->responses = g_list_append
+    (fixture->dataclient->responses, response);
+
+  g_mutex_lock (&fixture->dataclient->process_response_mutex);
+  
+  val = fixture->iface->transaction_get (tx, fixture->store, "foo", 4, NULL);  
+
+  g_cond_wait (&fixture->dataclient->process_response_cond,
+	       &fixture->dataclient->process_response_mutex);
+
+  g_mutex_unlock (&fixture->dataclient->process_response_mutex);
+
+  release_key (fixture->dataclient, "test", "test", key_bytes);
+  
+  g_assert_cmpint (g_list_length (fixture->dataclient->released_keys), ==, 0);
+
+  fixture->iface->transaction_rollback (tx);
+
+  g_assert_cmpint (g_list_length (fixture->dataclient->released_keys), ==, 1);
+
+  free (val);
+  
+  free_response (response);
+  g_bytes_unref (key_bytes);
+  g_bytes_unref (value_bytes);
+}
+
+static void
+test_range_lock_release_eviction (dataclient_storage_fixture *fixture,
+				  gconstpointer user_data)
+{
+  GBytes *key_bytes = g_bytes_new_static ("foo", 4);
+  dataclient_storage_response *response = create_success_response (key_bytes);
+  gzochid_storage_transaction *tx = fixture->iface->transaction_begin
+    (fixture->storage_context);
+  char *key = NULL;
+  
+  fixture->dataclient->responses = g_list_append
+    (fixture->dataclient->responses, response);
+
+  g_mutex_lock (&fixture->dataclient->process_response_mutex);
+
+  key = fixture->iface->transaction_first_key (tx, fixture->store, NULL);  
+  
+  g_cond_wait (&fixture->dataclient->process_response_cond,
+	       &fixture->dataclient->process_response_mutex);
+
+  g_mutex_unlock (&fixture->dataclient->process_response_mutex);
+  
+  release_key_range (fixture->dataclient, "test", "test", NULL, NULL);
+
+  g_assert_cmpint (g_list_length (fixture->dataclient->released_keys), ==, 0);
+
+  fixture->iface->transaction_rollback (tx);
+
+  g_assert_cmpint (g_list_length (fixture->dataclient->released_keys), ==, 1);
+
+  free (key);
+  
+  free_response (response);
+  g_bytes_unref (key_bytes);
+}
+
+static void
+test_cache_store_consistency (dataclient_storage_fixture *fixture,
+			      gconstpointer user_data)
+{
+  GBytes *key_bytes = g_bytes_new_static ("foo", 4);
+  GBytes *value1_bytes = g_bytes_new_static ("bar", 4);
+  GBytes *value2_bytes = g_bytes_new_static ("baz", 4);
+  GBytes *value3_bytes = g_bytes_new_static ("quux", 4);
+
+  char *value = NULL;
+  size_t value_len = 0;
+  GBytes *value_bytes = NULL;
+  
+  gzochid_storage_transaction *tx = fixture->iface->transaction_begin
+    (fixture->storage_context);
+
+  dataclient_storage_response *response1 =
+    create_success_response (value1_bytes);
+  dataclient_storage_response *response2 =
+    create_success_response (value1_bytes);
+  dataclient_storage_response *response3 =
+    create_success_response (value3_bytes);
+  
+  fixture->dataclient->responses = g_list_append
+    (g_list_append (fixture->dataclient->responses, response1), response2);
+
+  g_mutex_lock (&fixture->dataclient->process_response_mutex);
+
+  value = fixture->iface->transaction_get (tx, fixture->store, "foo", 4, NULL);
+  g_cond_wait (&fixture->dataclient->process_response_cond,
+	       &fixture->dataclient->process_response_mutex);
+  
+  g_mutex_unlock (&fixture->dataclient->process_response_mutex);
+  
+  free (value);
+  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "baz", 4);
+
+  fixture->iface->transaction_prepare (tx);
+  fixture->iface->transaction_commit (tx);
+
+  /* One release for the read lock, the other for the write lock. */
+  
+  release_key (fixture->dataclient, "test", "test", key_bytes);
+  release_key (fixture->dataclient, "test", "test", key_bytes);
+  
+  tx = fixture->iface->transaction_begin (fixture->storage_context);
+  
+  fixture->dataclient->responses = g_list_append
+    (fixture->dataclient->responses, response3);
+
+  g_mutex_lock (&fixture->dataclient->process_response_mutex);
+
+  value = fixture->iface->transaction_get
+    (tx, fixture->store, "foo", 4, &value_len);
+  g_cond_wait (&fixture->dataclient->process_response_cond,
+	       &fixture->dataclient->process_response_mutex);  
+  
+  g_mutex_unlock (&fixture->dataclient->process_response_mutex);
+
+  fixture->iface->transaction_rollback (tx);
+  
+  value_bytes = g_bytes_new_static (value, value_len);
+  g_assert (g_bytes_equal (value3_bytes, value_bytes));
+  
+  free_response (response1);
+  free_response (response2);
+  free_response (response3);
+  
+  g_bytes_unref (key_bytes);
+  g_bytes_unref (value1_bytes);
+  g_bytes_unref (value2_bytes);
+  g_bytes_unref (value3_bytes);
+  g_bytes_unref (value_bytes);
+  
+  free (value);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1281,5 +1604,19 @@ main (int argc, char *argv[])
      dataclient_storage_fixture, NULL, dataclient_storage_fixture_setup,
      test_next_key_uncached_timeout, dataclient_storage_fixture_teardown);
 
+  g_test_add
+    ("/storage-dataclient/lock-release/eviction", dataclient_storage_fixture,
+     NULL, dataclient_storage_fixture_setup, test_lock_release_eviction,
+     dataclient_storage_fixture_teardown);
+  g_test_add
+    ("/storage-dataclient/range-lock-release/eviction",
+     dataclient_storage_fixture, NULL, dataclient_storage_fixture_setup,
+     test_range_lock_release_eviction, dataclient_storage_fixture_teardown);
+
+  g_test_add
+    ("/storage-dataclient/consistency/cache-store",
+     dataclient_storage_fixture, NULL, dataclient_storage_fixture_setup,
+     test_cache_store_consistency, dataclient_storage_fixture_teardown);
+  
   return g_test_run ();
 }
